'Set up the screen
'bOrientation = Origin And Direction: Width x Height
'                                     0 = 320 x 240  
'                                     1 = 240 x 320 
'                                     2 = 320 x 240 inverse 
'                                     3 = 240 x 320 inverse  
'Origin is always bottom left corner and direction is left to right, bottom - up

Proc TftSetUp(bOrientation As Byte) 
    'HW TFT Reset
        $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
            $ifdef T_CS
                PinSet T_CS
            $endif
            PinSet TFT_CS : PinClear TFT_RESET     
            DelayMS 100 : PinSet TFT_RESET : DelayMS 100 
        $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
            $if TFT_COM = PAR_2x8b
                Output TftDataLowByte
                Output TftDataHighByte
            $else
                Output TftDataWord                    
            $endif
            $ifdef T_CS
                PinSet T_CS
            $endif
            PinSet TFT_CS : PinSet TFT_WR : PinSet TFT_RD
            PinClear TFT_RESET : DelayMS 100 : PinSet TFT_RESET : DelayMS 100
        $endif
        
        'Initialization sequence
        Tft4bWriteCommand(0xef, 0x03, 0x80, 0x02) 
        'Power control B
        Tft4bWriteCommand(0xcf, 0x00, 0xc1, 0x30) 
        
        $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
            'Power on sequence control
            PinClear TFT_SCLK : PinClear TFT_DC
            PinClear TFT_CS
                TftSend(0xed)
            PinSet TFT_DC
                TftSend(0x64)
                TftSend(0x03)
                TftSend(0x12)
                TftSend(0x81)
            PinSet TFT_CS
        $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
            PinClear TFT_CS 
            PinClear TFT_DC 
                TftDataLowByte = 0xED
                PinClear TFT_WR : PinSet TFT_WR
            PinSet TFT_DC 
                TftDataLowByte = 0x64  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x03  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x12  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x81  
                PinClear TFT_WR : PinSet TFT_WR
            PinSet TFT_CS 
        $endif
        
        'Driver timing control A
        Tft4bWriteCommand(0xe8, 0x85, 0x00, 0x78)
        
        $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
            'Power control A
            PinClear TFT_SCLK : PinClear TFT_DC
            PinClear TFT_CS
                TftSend(0xcb)
            PinSet TFT_DC
                TftSend(0x39)
                TftSend(0x2c)
                TftSend(0x00)
                TftSend(0x34)
                TftSend(0x02)
            PinSet TFT_CS
        $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
            PinClear TFT_CS 
            PinClear TFT_DC 
                TftDataLowByte = 0xCB
                PinClear TFT_WR : PinSet TFT_WR
            PinSet TFT_DC 
                TftDataLowByte = 0x39  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x2c  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x00  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x34  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x02  
                PinClear TFT_WR : PinSet TFT_WR
            PinSet TFT_CS   
        $endif
        
        'Pump ratio control
        Tft2bWriteCommand(0xf7, 0x20)
        'Driver timing control B
        Tft3bWriteCommand(0xea, 0x00, 0x00)
        'Power Control 1, VRH[5:0]
        Tft2bWriteCommand(0xc0, 0x23)
        'Power Control 2, SAP[2:0], BT[3:0]
        Tft2bWriteCommand(0xc1, 0x10)
        'VCM Control 1
        Tft3bWriteCommand(0xc5, 0x2b, 0x2b)
        'VCM Control 2
        Tft2bWriteCommand(0xc7, 0xc0)
        'Screen orientation 
        TftSetOrientation(bOrientation)  
        'Pixel Format 5-6-5
        Tft2bWriteCommand(0x3a, %01010101)
        'FRMCTR1
        Tft3bWriteCommand(0xb1, 0x00, 0x13)
        'Display Function Control
        Tft4bWriteCommand(0xb6, 0x08, 0x82, 0x27)
        '3Gamma Function Disable
        Tft2bWriteCommand(0xf2, 0x00)
        'Gamma Curve Selected
        Tft3bWriteCommand(0x26, 0x00, 0x1b)
        'Entry Mode Set
        Tft2bWriteCommand(0xb7, 0x07)
        
        $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
            'Set Gamma (+ polarity)
            PinClear TFT_SCLK : PinClear TFT_DC
            PinClear TFT_CS
                TftSend(0xe0)
            PinSet TFT_DC
                TftSend(0x0f)
                TftSend(0x31)
                TftSend(0x2b)
                TftSend(0x0c)
                TftSend(0x0e)
                TftSend(0x08)
                TftSend(0x4e)
                TftSend(0xf1)
                TftSend(0x37)
                TftSend(0x07)
                TftSend(0x10)
                TftSend(0x03)
                TftSend(0x0e)
                TftSend(0x09)
                TftSend(0x00)
            PinSet TFT_CS
            
            'Set Gamma (- polarity)
            PinClear TFT_SCLK : PinClear TFT_DC
            PinClear TFT_CS
                TftSend(0xe1)
            PinSet TFT_DC
                TftSend(0x00)
                TftSend(0x0e)
                TftSend(0x14)
                TftSend(0x03)
                TftSend(0x11)
                TftSend(0x07)
                TftSend(0x31)
                TftSend(0xc1)
                TftSend(0x48)
                TftSend(0x08)
                TftSend(0x0f)
                TftSend(0x0c)
                TftSend(0x31)
                TftSend(0x36)
                TftSend(0x0f)
            PinSet TFT_CS
        $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
            PinClear TFT_CS 
            PinClear TFT_DC 
                TftDataLowByte = 0xE0
                PinClear TFT_WR : PinSet TFT_WR
            PinSet TFT_DC 
                TftDataLowByte = 0x0F  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x31  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x2b  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x0c  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x0e  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x0e  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x08  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x4e  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0xf1  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x37  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x07  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x10  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x03  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x0e  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x09  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x00  
            PinSet TFT_CS 
            
            PinClear TFT_CS 
            PinClear TFT_DC 
                TftDataLowByte = 0xE1
                PinClear TFT_WR : PinSet TFT_WR
            PinSet TFT_DC 
                TftDataLowByte = 0x00  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x0e  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x14  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x03  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x11  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x07  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x31  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0xc1  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x48  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x08  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x0f  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x0c  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x31  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x36  
                PinClear TFT_WR : PinSet TFT_WR
                TftDataLowByte = 0x0f  
            PinSet TFT_CS  
        $endif
        
        'Sleep Out
        Tft1bWriteCommand(0x11)
        'Normal Display Mode On
        Tft1bWriteCommand(0x13)
        'Display ON
        Tft1bWriteCommand(0x29)   
EndProc

'Changes the screen point of origin
'bOrientation = Origin And Direction: Width x Height
'                                     0 = 320 x 240  
'                                     1 = 240 x 320 
'                                     2 = 320 x 240 inverse 
'                                     3 = 240 x 320 inverse  

Proc TftSetOrientation(bOrientation As Byte)
    Select bOrientation
        Case 0
            Tft2bWriteCommand(0x36, %10111000)     
        Case 1
            Swap wTftScreenWidth, wTftScreenHeight  
            Swap wTftWidthOffset, wTftHeightOffset 
            Tft2bWriteCommand(0x36, %00011000)    
        Case 2
            Tft2bWriteCommand(0x36, %01111000)
        Case 3
            Swap wTftScreenWidth, wTftScreenHeight 
            Swap wTftWidthOffset, wTftHeightOffset  
            Tft2bWriteCommand(0x36, %11011000) 
    EndSelect
EndProc
