Proc ______TFT_PRINT_SND_____()
EndProc

$ifndef FontSmooth
    $define FontSmooth 0
$endif

'Print character from font using forward lookup with parameters:
'bChar = character to print
'bFontSize = font size: 1 = nominal size, 2,3,4 ... multiple of the nominal size
'wX_Pos = x position for the lower left corner of this character
'wY_Pos = y position for the lower left corner of this character   
'pPosition = lower left corner for character position: 
'                                                    0 = absolute, actual pixel position on TFT screen (example wX_Pos = 152, wY_Pos = 86)   
'                                                    1 = relative, relative to the virtual division of the TFT screen by FontPixels and FontSpace to columns and rows 
'                                                        (example wX_Pos = 5, wY_Pos = 2 would mean actual position, if FontSpace = 9, x = 5*9 = 45 ; y = 2*9 = 18)  
'wForeColor = character color (pixels, forming the character use this color)
'wBackColor = background color (pixels from the FontSpace*FontSpace box forming each character space will use this color)
'pTransparent = transparent font. Background color (wBackColor) will be ignored and only the pixels will be printed, this however is slower than using background color
'Example: TftPrintSndChar("G", 2, 1, 1, 0, WHITE, BLACK, 0) --> 
'Print character "G" ; font size 2x ; at coordinates x=1 ; y=1 ; coordinates are absolute ; font color White ; Background color Black ; not transparent font, no smooth resize

Proc TftPrintSndChar(bChar As Byte, bFontSize As Byte, wX_Pos As Word, wY_Pos As Word, pPosition As Bit, wForeColor As Word, wBackColor As Word, pTransparent As Bit)
$if FontSmooth = 1
    $if FontMaxPixels < 9
        Global Dim aTftPrintArray[8] As Byte Shared
    $elseif FontMaxPixels = 16 
        Global Dim aTftPrintArray[16] As Word Shared
    $elseif FontMaxPixels > 16
        Global Dim aTftPrintArray[32] As Dword Shared
    $endif
   
    Global Dim bFontSpace As Byte Shared
    Global Dim bFontPixels As Byte Shared
    Global Dim bFontOffset As Byte Shared
    Global Dim dwFontAddress As Dword Shared
    
    Dim OffsetX As Word     At dwDummy1.Word0
    Dim OffsetY As Word     At dwDummy1.Word1
    Dim RelativeX As Byte
    Dim RelativeXOffset1 As Byte
    Dim RelativeXOffset2 As Byte
    Dim RelativeY As Byte
    Dim RelativeYOffset As Byte
    Dim yCounter As Byte
    Dim xCounter As Byte
    Dim iCounter As Byte
    Dim jCounter As Byte
    Dim bCounter As Byte
    Dim pCounter As Byte
    Dim bPixelBytes As Byte
    Dim wTableOffset As Word
    Dim bTemp As Byte
    Dim pBit1 As Byte
    Dim pBit2 As Byte
    Dim pBit3 As Byte
    Dim pBit4 As Byte
    Dim dwTemp1 As Dword    At dwDummy2
    Dim dwTemp2 As Dword    At dwDummy3
    
    If bFontSize = 0 Then bFontSize = 1
    
    If pPosition = 0 Then
        OffsetX = wX_Pos 
        OffsetY = wY_Pos 
    Else
        OffsetX = wX_Pos * bFontPixels * bFontSize 
        OffsetY = wY_Pos * bFontSpace * bFontSize
    EndIf 
    
    If bFontSize = 1 And pTransparent = 0 Then
        If bChar = 0 Then bChar = $20
        bChar = bChar - bFontOffset
    
        'bPixelBytes = 1 + ((bFontPixels - 1) / 8)
        bPixelBytes = bFontPixels - 1
        bPixelBytes = bPixelBytes >> 3
        Inc bPixelBytes 
    
        wTableOffset = bChar * bPixelBytes * bFontSpace
        wTableOffset = wTableOffset + (bPixelBytes * bFontSpace) 
        iCounter = 0 
    
        $if TftPixelCheck = 1
            If OffsetX < TftScreenWidth And OffsetY < TftScreenHeight Then
                wTempHorizontal = OffsetX + bFontPixels - 1 : wTempVertical = OffsetY + bFontSpace - 1 : pOutOfScreen = 0
                If wTempHorizontal >= TftScreenWidth Then 
                    wTempHorizontal = TftScreenWidth - 1
                    pOutOfScreen = 1
                EndIf
                If wTempVertical >= TftScreenHeight Then 
                    wTempVertical = TftScreenHeight - 1 
                    pOutOfScreen = 1
                EndIf
                TftSetWindow(OffsetX, wTempHorizontal, OffsetY, wTempVertical)
            Else
                pOutOfScreen = 1
                ExitProc   
            EndIf 
        $else
            TftSetWindow(OffsetX, OffsetX + bFontPixels - 1, OffsetY, OffsetY + bFontSpace - 1)
        $endif
        TftInitiateDataRam()
        
        Do
            For bCounter = bPixelBytes DownTo 1 Step -1
                bTemp = CRead8 dwFontAddress[wTableOffset - bCounter]
                For pCounter = 7 DownTo 0 Step -1
                    pBit1 = GetBit bTemp, pCounter
                    If pBit1 = 1 Then
                        $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
                            $if TFT_IC = ILI9488 
                                Rgb16ToRgb18(wForeColor) 
                                TftSend(dColor.Byte2)
                                TftSend(dColor.Byte1)
                                TftSend(dColor.Byte0)  
                            $else
                                TftSend(wForeColor.Byte1)
                                TftSend(wForeColor.Byte0)  
                            $endif  
                        $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
                            TftSend(wForeColor)
                        $endif
                    Else
                        $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
                            $if TFT_IC = ILI9488 
                                Rgb16ToRgb18(wBackColor) 
                                TftSend(dColor.Byte2)
                                TftSend(dColor.Byte1)
                                TftSend(dColor.Byte0)  
                            $else
                                TftSend(wBackColor.Byte1)
                                TftSend(wBackColor.Byte0)  
                            $endif  
                        $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
                            TftSend(wBackColor)
                        $endif
                    EndIf
                Next
            Next
            Inc iCounter 
            wTableOffset = wTableOffset - bPixelBytes
        Loop Until iCounter = bFontSpace 
        
        TftStopDataRam()
        ExitProc
    EndIf
    
    If bFontPixels <> bFontSpace Or bFontPixels > 32 Then
        If bChar = 0 Then bChar = $20
        bChar = bChar - bFontOffset
    
        'bPixelBytes = 1 + ((bFontPixels - 1) / 8)
        bPixelBytes = bFontPixels - 1
        bPixelBytes = bPixelBytes >> 3
        Inc bPixelBytes 
    
        wTableOffset = bChar * bPixelBytes * bFontSpace
        wTableOffset = wTableOffset + (bPixelBytes * bFontSpace) 
        iCounter = 0
        
        Do
            RelativeYOffset = iCounter * bFontSize
        
            For bCounter = bPixelBytes DownTo 1 Step -1 
                bTemp = CRead8 dwFontAddress[wTableOffset - bCounter] 
                RelativeXOffset1 = (bPixelBytes - bCounter) * bFontSize
                RelativeXOffset1 = RelativeXOffset1 << 3
                jCounter = 0 
                For pCounter = 7 DownTo 0 Step -1
                    pBit1 = GetBit bTemp, pCounter
                    RelativeXOffset2 = jCounter * bFontSize
                    For yCounter = 0 To bFontSize - 1               'Print Y-direction
                        RelativeY = RelativeYOffset + yCounter
                        For xCounter = 0 To bFontSize - 1           'Print X-direction
                            RelativeX = RelativeXOffset2 + xCounter + RelativeXOffset1 
                            If pBit1 = 1 Then
                                TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor)
                            Else
                                If pTransparent = 0 Then
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wBackColor)
                                EndIf
                            EndIf
                        Next
                    Next
                    Inc jCounter
                Next
            Next
        
            Inc iCounter 
            wTableOffset = wTableOffset - bPixelBytes
        Loop Until iCounter = bFontSpace 
        ExitProc
    Else
        TftLookSndFont(bChar)

        For yCounter = 0 To bFontPixels - 1                                          
            dwTemp1 = aTftPrintArray[yCounter]    
            RelativeY = yCounter * bFontSize                                  
            For xCounter = 0 To bFontPixels - 1  
                RelativeX = xCounter * bFontSize                                    
                pBit1 = GetBit dwTemp1, xCounter                                     
                For jCounter = 0 To bFontSize - 1                                   
                    For iCounter = 0 To bFontSize - 1                               
                        If pBit1 = 1 Then
                            TftPixel(OffsetX + RelativeX + iCounter, OffsetY + RelativeY + jCounter, wForeColor)
                        Else
                            If pTransparent = 0 Then
                                TftPixel(OffsetX + RelativeX + iCounter, OffsetY + RelativeY + jCounter, wBackColor)
                            EndIf
                        EndIf
                    Next
                Next
            Next
        Next
    EndIf 
    
    If pSmoothResize = 0 Then ExitProc
    
    If bFontSize > 1 And bFontPixels = bFontSpace And bFontSpace < 33 Then 
        For yCounter = 0 To bFontPixels - 1                                         
            dwTemp1 = aTftPrintArray[yCounter]                                      
            For xCounter = 0 To bFontPixels - 1                                     
                pBit1 = GetBit dwTemp1, xCounter                                   
                If yCounter > 1 Then
                    If xCounter = 0 Then
                        dwTemp2 = aTftPrintArray[yCounter - 1] 
                        pBit2 = GetBit dwTemp2, xCounter + 1
                        If pBit1 = 1 And pBit2 = 1 Then
                            For iCounter = 0 To bFontSize - 2                                           
                                For jCounter = 0 To bFontSize - 2 - iCounter                           
                                    RelativeX = bFontSize + iCounter
                                    RelativeY = yCounter * bFontSize + jCounter
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor)
                                    RelativeX = bFontSize - 1 - iCounter
                                    RelativeY = yCounter * bFontSize - 1 - jCounter
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor) 
                                Next
                            Next
                        EndIf
                    ElseIf xCounter = bFontPixels - 1  Then
                        dwTemp2 = aTftPrintArray[yCounter - 1] 
                        pBit2 = GetBit dwTemp2, xCounter - 1
                        If pBit1 = 1 And pBit2 = 1 Then
                            For iCounter = 0 To bFontSize - 2                                           
                                For jCounter = 0 To bFontSize - 2 - iCounter                            
                                    RelativeX = xCounter * bFontSize - 1 - iCounter
                                    RelativeY = yCounter * bFontSize + jCounter
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor)
                                    RelativeX = xCounter * bFontSize + iCounter
                                    RelativeY = yCounter * bFontSize - 1 - jCounter
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor)
                                Next
                            Next
                        EndIf
                    Else
                        dwTemp2 = aTftPrintArray[yCounter - 1] 
                        pBit2 = GetBit dwTemp2, xCounter - 1
                        If pBit1 = 1 And pBit2 = 1 Then
                            For iCounter = 0 To bFontSize - 2                                           
                                For jCounter = 0 To bFontSize - 2 - iCounter                            
                                    RelativeX = xCounter * bFontSize - 1 - iCounter
                                    RelativeY = yCounter * bFontSize + jCounter
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor)
                                    RelativeX = xCounter * bFontSize + iCounter
                                    RelativeY = yCounter * bFontSize - 1 - jCounter
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor)
                                Next
                            Next
                        EndIf
                        pBit2 = GetBit dwTemp2, xCounter + 1
                        If pBit1 = 1 And pBit2 = 1 Then
                            For iCounter = 0 To bFontSize - 2                                          
                                For jCounter = 0 To bFontSize - 2 - iCounter                            
                                    RelativeX = xCounter * bFontSize + bFontSize + iCounter
                                    RelativeY = yCounter * bFontSize + jCounter
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor) 
                                    RelativeX = xCounter * bFontSize + bFontSize - 1 - iCounter
                                    RelativeY = yCounter * bFontSize - 1 - jCounter
                                    TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor)
                                Next
                            Next
                        EndIf
                    EndIf
                EndIf
            Next
        Next
    EndIf 
$else
    Global Dim bFontSpace As Byte Shared
    Global Dim bFontPixels As Byte Shared
    Global Dim bFontOffset As Byte Shared
    Global Dim dwFontAddress As Dword Shared
    
    Dim OffsetX As Word             At dwDummy1.Word0
    Dim OffsetY As Word             At dwDummy1.Word1
    Dim RelativeX As Byte
    Dim RelativeXOffset1 As Byte
    Dim RelativeXOffset2 As Byte
    Dim RelativeY As Byte
    Dim RelativeYOffset As Byte
    Dim wTableOffset As Word        At dwDummy2.Word0
    Dim bPixelBytes As Byte
    Dim bTemp As Byte
    Dim pBit1 As Byte
    Dim pCounter As Byte            At dwDummy2.Byte2
    Dim bCounter As Byte            At dwDummy2.Byte3
    Dim xCounter As Byte            At dwDummy3.Byte0
    Dim yCounter As Byte            At dwDummy3.Byte1
    Dim iCounter As Byte            At dwDummy3.Byte2
    Dim jCounter As Byte            At dwDummy3.Byte3
    

    
    If bFontSize = 0 Then bFontSize = 1
    
    If pPosition = 0 Then
        OffsetX = wX_Pos  
        OffsetY = wY_Pos
    Else
        OffsetX = wX_Pos * bFontPixels * bFontSize 
        OffsetY = wY_Pos * bFontSpace * bFontSize
    EndIf
    
    If bChar = 0 Then bChar = $20
    bChar = bChar - bFontOffset
    
    'bPixelBytes = 1 + ((bFontPixels - 1) / 8)
    bPixelBytes = bFontPixels - 1
    bPixelBytes = bPixelBytes >> 3
    Inc bPixelBytes 
    
    wTableOffset = bChar * bPixelBytes * bFontSpace
    wTableOffset = wTableOffset + (bPixelBytes * bFontSpace) 
    iCounter = 0 
    
    If bFontSize = 1 And pTransparent = 0 Then
        $if TftPixelCheck = 1
            If OffsetX < TftScreenWidth And OffsetY < TftScreenHeight Then
                wTempHorizontal = OffsetX + bFontPixels - 1 : wTempVertical = OffsetY + bFontSpace - 1 : pOutOfScreen = 0
                If wTempHorizontal >= TftScreenWidth Then 
                    wTempHorizontal = TftScreenWidth - 1
                    pOutOfScreen = 1
                EndIf
                If wTempVertical >= TftScreenHeight Then 
                    wTempVertical = TftScreenHeight - 1 
                    pOutOfScreen = 1
                EndIf  
                TftSetWindow(OffsetX, wTempHorizontal, OffsetY, wTempVertical)
            Else
                pOutOfScreen = 1
                ExitProc                  
            EndIf
        $else
            TftSetWindow(OffsetX, OffsetX + bFontPixels - 1, OffsetY, OffsetY + bFontSpace - 1)
        $endif
        TftInitiateDataRam()
        
        Do
            For bCounter = bPixelBytes DownTo 1 Step -1
                bTemp = CRead8 dwFontAddress[wTableOffset - bCounter]
                For pCounter = 7 DownTo 0 Step -1
                    pBit1 = GetBit bTemp, pCounter
                    If pBit1 = 1 Then
                        $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
                            $if TFT_IC = ILI9488 
                                Rgb16ToRgb18(wForeColor) 
                                TftSend(dColor.Byte2)
                                TftSend(dColor.Byte1)
                                TftSend(dColor.Byte0)  
                            $else
                                TftSend(wForeColor.Byte1)
                                TftSend(wForeColor.Byte0)  
                            $endif  
                        $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
                            TftSend(wForeColor)
                        $endif
                    Else
                        $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
                            $if TFT_IC = ILI9488 
                                Rgb16ToRgb18(wBackColor) 
                                TftSend(dColor.Byte2)
                                TftSend(dColor.Byte1)
                                TftSend(dColor.Byte0)  
                            $else
                                TftSend(wBackColor.Byte1)
                                TftSend(wBackColor.Byte0)  
                            $endif 
                        $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
                            TftSend(wBackColor)
                        $endif
                    EndIf
                Next
            Next
            Inc iCounter 
            wTableOffset = wTableOffset - bPixelBytes
        Loop Until iCounter = bFontSpace 
        
        TftStopDataRam()
        ExitProc
    EndIf
    
    Do
        RelativeYOffset = iCounter * bFontSize
        
        For bCounter = bPixelBytes DownTo 1 Step -1 
            bTemp = CRead8 dwFontAddress[wTableOffset - bCounter] 
            RelativeXOffset1 = (bPixelBytes - bCounter) * bFontSize
            RelativeXOffset1 = RelativeXOffset1 << 3
            jCounter = 0 
            For pCounter = 7 DownTo 0 Step -1
                pBit1 = GetBit bTemp, pCounter
                RelativeXOffset2 = jCounter * bFontSize
                For yCounter = 0 To bFontSize - 1               'Print Y-direction
                    RelativeY = RelativeYOffset + yCounter
                    For xCounter = 0 To bFontSize - 1           'Print X-direction
                        RelativeX = RelativeXOffset2 + xCounter + RelativeXOffset1 
                        If pBit1 = 1 Then
                            TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wForeColor)
                        Else
                            If pTransparent = 0 Then
                                TftPixel(OffsetX + RelativeX, OffsetY + RelativeY, wBackColor)
                            EndIf
                        EndIf
                    Next
                Next
                Inc jCounter
            Next
        Next
        
        Inc iCounter 
        wTableOffset = wTableOffset - bPixelBytes
    Loop Until iCounter = bFontSpace     
$endif
EndProc

'Print a string using standard font with parameters:
'sString = null-terminated string to print up to 72 characters
'bFontSize = font size: 1 = nominal size, 2,3,4 ... multiple of the nominal size 
'wX_Pos = x position for the lower left corner of this character
'wY_Pos = y position for the lower left corner of this character   
'pPosition = lower left corner for character position: 
'                                                    0 = absolute, actual pixel position on TFT screen (example wX_Pos = 152, wY_Pos = 86)   
'                                                    1 = relative, relative to the virtual division of the TFT screen by FontSpace to columns and rows 
'                                                        (example wX_Pos = 5, wY_Pos = 2 would mean actual position, if FontSpace = 9, x = 5*9 = 45 ; y = 2*9 = 18)  
'wForeColor = character color (pixels, forming the character use this color)
'wBackColor = background color (pixels from the 10x10 box forming each character space will use this color)
'pTransparent = transparent font. Background color (wBackColor) will be ignored and only the pixels will be printed, this however is slower than using background color
'Example: TftPrintSndString("TEST", 2, 1, 1, 0, WHITE, BLACK, 0) --> 
'Print string "TEST" ; font size 2x ; at coordinates x=1 ; y=1 ; coordinates are absolute ; font color White ; Background color Black ; not transparent font

Proc TftPrintSndString(sString As sTftString, bFontSize As Byte, wX_Pos As Word, wY_Pos As Word, pPosition As Bit, wForeColor As Word, wBackColor As Word, pTransparent As Bit)
    Dim bTemp As Byte
    
    If bFontSize = 0 Then bFontSize = 1
    
    For bTemp = 0 To 71
        If sString[bTemp] <> 0 Then
            If pPosition = 0 Then
                TftPrintSndChar(sString[bTemp], bFontSize, wX_Pos + bTemp * bFontPixels * bFontSize, wY_Pos, pPosition, wForeColor, wBackColor, pTransparent)
            Else
                TftPrintSndChar(sString[bTemp], bFontSize, wX_Pos + bTemp, wY_Pos, pPosition, wForeColor, wBackColor, pTransparent)
            EndIf
        Else
            Break
        EndIf
    Next    
EndProc

'Print a signed integer with parameters:
'sdInteger = signed integer to print
'bDigits = number of digits to print, if bDigits is larger than the number of significant digits, leading zeroes will be printed
'                                     if bDigits is smaller than the number of significant digits, only bDigits digits will be printed
'                                     if bDigits is 0 then the significant digits will be automatically printed 
'bFontSize = font size: 1 = nominal size, 2,3,4 ... multiple of the nominal size 
'wX_Pos = x position for the lower left corner of this character
'wY_Pos = y position for the lower left corner of this character   
'pPosition = lower left corner for character position: 
'                                                    0 = absolute, actual pixel position on TFT screen (example wX_Pos = 152, wY_Pos = 86)   
'                                                    1 = relative, relative to the virtual division of the TFT screen by FontSpace to columns and rows 
'                                                        (example wX_Pos = 5, wY_Pos = 2 would mean actual position, if FontSpace = 9, x = 5*9 = 45 ; y = 2*9 = 18)  
'wForeColor = character color (pixels, forming the character use this color)
'wBackColor = background color (pixels from the 10x10 box forming each character space will use this color)
'pTransparent = transparent font. Background color (wBackColor) will be ignored and only the pixels will be printed, this however is slower than using background color
'Example: TftPrintSndInteger(-12345, 7, 2, 1, 1, 0, WHITE, BLACK, 0) --> 
'Print integer -12345 ; 7 significant digits ; font size 2x ; at coordinates x=1 ; y=1 ; coordinates are absolute ; font color White ; Background color Black ; not transparent font

Proc TftPrintSndInteger(sdInteger As SDword, bDigits As Byte, bFontSize As Byte, wX_Pos As Word, wY_Pos As Word, pPosition As Bit, wForeColor As Word, wBackColor As Word, pTransparent As Bit)
    Select bDigits
        Case 1
            sTftString = Str$(SDec1 sdInteger)
        Case 2
            sTftString = Str$(SDec2 sdInteger)
        Case 3
            sTftString = Str$(SDec3 sdInteger)
        Case 4
            sTftString = Str$(SDec4 sdInteger)
        Case 5
            sTftString = Str$(SDec5 sdInteger)
        Case 6
            sTftString = Str$(SDec6 sdInteger)
        Case 7
            sTftString = Str$(SDec7 sdInteger)
        Case 8
            sTftString = Str$(SDec8 sdInteger)
        Case 9
            sTftString = Str$(SDec9 sdInteger)
        Case 10
            sTftString = Str$(SDec10 sdInteger)
        Case Else
            sTftString = Str$(SDec sdInteger)
    EndSelect
    TftPrintSndString(sTftString, bFontSize, wX_Pos, wY_Pos, pPosition, wForeColor, wBackColor, pTransparent)
EndProc

'Print a signed decimal with parameters:
'fDec = signed decimal to print
'bSigDigits = number of significant digits to print, if bSigDigits is larger than the number of significant digits, leading zeroes will be printed
'                                                    if bSigDigits is smaller than the number of significant digits, only bSigDigits digits will be printed
'                                                    if bSigDigits = 0 then the number of significant digits will be automatically printed 
'bDecDigits = number of decimal digits to print, if bDecDigits is larger than the number of decimal digits, trailing zeroes will be printed
'                                                if bDecDigits is smaller than the number of decimal digits, only bDecDigits digits will be printed
'                                                if bDecDigits is 0 then a signed integer will be printed 
'bFontSize = font size: 1 = nominal size, 2,3,4 ... multiple of the nominal size 
'wX_Pos = x position for the lower left corner of this character
'wY_Pos = y position for the lower left corner of this character   
'pPosition = lower left corner for character position: 
'                                                    0 = absolute, actual pixel position on TFT screen (example wX_Pos = 152, wY_Pos = 86)   
'                                                    1 = relative, relative to the virtual division of the TFT screen by FontSpace to columns and rows 
'                                                        (example wX_Pos = 5, wY_Pos = 2 would mean actual position, if FontSpace = 9, x = 5*9 = 45 ; y = 2*9 = 18) 
'wForeColor = character color (pixels, forming the character use this color)
'wBackColor = background color (pixels from the 10x10 box forming each character space will use this color)
'pTransparent = transparent font. Background color (wBackColor) will be ignored and only the pixels will be printed, this however is slower than using background color
'Example: TftPrintSndDecimal(-12345.678, 7, 4, 2, 1, 1, 0, WHITE, BLACK, 0) --> 
'Print decimal -12345.678 ; 7 significant digits ; 4 decimal digits ; font size 2x ; at coordinates x=1 ; y=1 ; coordinates are absolute ; font color White ; Background color Black ; not transparent font

Proc TftPrintSndDecimal(fDec As Float, bSigDigits As Byte, bDecDigits As Byte, bFontSize As Byte, wX_Pos As Word, wY_Pos As Word, pPosition As Bit, wForeColor As Word, wBackColor As Word, pTransparent As Bit)
    Dim sdwTemp As SDword
    
    sdwTemp = fDec
    
    If bDecDigits = 0 Then
        TftPrintSndInteger(sdwTemp, bSigDigits, bFontSize, wX_Pos, wY_Pos, pPosition, wForeColor, wBackColor, pTransparent)
        ExitProc
    EndIf
    
    Clear sTftString
    sdwTemp = Abs(fDec)
    If fDec < 0 Then
        sTftString = "-"
    EndIf

    Select bSigDigits
        Case 1
            sTftString = sTftString + Str$(SDec1 sdwTemp) + "."
        Case 2
            sTftString = sTftString + Str$(SDec2 sdwTemp) + "."
        Case 3
            sTftString = sTftString + Str$(SDec3 sdwTemp) + "."
        Case 4
            sTftString = sTftString + Str$(SDec4 sdwTemp) + "."
        Case 5
            sTftString = sTftString + Str$(SDec5 sdwTemp) + "."
        Case 6
            sTftString = sTftString + Str$(SDec6 sdwTemp) + "."
        Case 7
            sTftString = sTftString + Str$(SDec7 sdwTemp) + "."
        Case 8
            sTftString = sTftString + Str$(SDec8 sdwTemp) + "."
        Case 9
            sTftString = sTftString + Str$(SDec9 sdwTemp) + "."
        Case 10
            sTftString = sTftString + Str$(SDec10 sdwTemp) + "."
        Case Else
            sTftString = sTftString + Str$(SDec sdwTemp) + "."
    EndSelect

    sdwTemp.Dword = Abs(fDec) 
    fDec = Abs(fDec) - sdwTemp.Dword 

    Select bDecDigits
        Case 2
            fDec = fDec * 100
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec2 sdwTemp.Dword)
        Case 3
            fDec = fDec * 1000
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec3 sdwTemp.Dword)
        Case 4
            fDec = fDec * 10000
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec4 sdwTemp.Dword)
        Case 5
            fDec = fDec * 100000
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec5 sdwTemp.Dword)
        Case 6
            fDec = fDec * 1000000
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec6 sdwTemp.Dword)
        Case 7
            fDec = fDec * 10000000
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec7 sdwTemp.Dword)
        Case 8
            fDec = fDec * 100000000
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec8 sdwTemp.Dword)
        Case 9
            fDec = fDec * 1000000000
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec9 sdwTemp.Dword)
        Case 10
            fDec = fDec * 10000000000
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec10 sdwTemp.Dword)
        Case Else
            fDec = fDec * 10
            sdwTemp.Dword = fDec
            sTftString = sTftString + Str$(Dec1 sdwTemp.Dword)
    EndSelect

    TftPrintSndString(sTftString, bFontSize, wX_Pos, wY_Pos, pPosition, wForeColor, wBackColor, pTransparent)
EndProc


'Read standard fixed size pixel font 

Proc TftLookSndFont(wChar As Word)
    Dim bTempCounter1 As Byte 
    Dim dwTemp1 As Dword
    
    If wChar = 0 Then wChar = $20
    wChar = wChar - bFontOffset
    Select bFontPixels
        Case 0 To 8
            For bTempCounter1 = 0 To bFontPixels - 1
                dwTemp1.Byte0 = CRead8 dwFontAddress[wChar * bFontPixels + bTempCounter1] 
                    dwTemp1.Byte0 = dwTemp1.Byte0 @ 8
                aTftPrintArray[bFontPixels - 1 - bTempCounter1] = dwTemp1.Byte0
            Next
        Case 9 To 16
            For bTempCounter1 = 0 To bFontPixels - 1 
                dwTemp1.Byte1 = CRead8 dwFontAddress[2 * wChar * bFontPixels + 2 * bTempCounter1 + 0]
                dwTemp1.Byte0 = CRead8 dwFontAddress[2 * wChar * bFontPixels + 2 * bTempCounter1 + 1]
                    dwTemp1.Word0 = dwTemp1.Word0 @ 16
                aTftPrintArray[bFontPixels - 1 - bTempCounter1] = dwTemp1.Word0 
            Next
        Case > 16
            For bTempCounter1 = 0 To bFontPixels - 1
                dwTemp1.Byte3 = CRead8 dwFontAddress[4 * wChar * bFontPixels + 4 * bTempCounter1 + 0]
                dwTemp1.Byte2 = CRead8 dwFontAddress[4 * wChar * bFontPixels + 4 * bTempCounter1 + 1]
                dwTemp1.Byte1 = CRead8 dwFontAddress[4 * wChar * bFontPixels + 4 * bTempCounter1 + 2]
                dwTemp1.Byte0 = CRead8 dwFontAddress[4 * wChar * bFontPixels + 4 * bTempCounter1 + 3]
                    dwTemp1 = dwTemp1 @ 32
                aTftPrintArray[bFontPixels - 1 - bTempCounter1] = dwTemp1   
            Next
    EndSelect
EndProc

'Select the standard font to be used as indexed table and set the parameters for this font   
'bSndFont = select font table
'pSmoothResize = if the standard font is being resized, then the pixelization will be visible, 
'                this will smooth the edges, but requires more computation and changes the font look to bold 

Proc TftSetSndFont(bSndFont As Byte, pSmooth As Bit)
    Global Dim bFontSpace As Byte Shared
    Global Dim bFontPixels As Byte Shared
    Global Dim bFontOffset As Byte Shared
    Global Dim dwFontAddress As Dword Shared
    Global Dim pSmoothResize As Byte Shared
    
    pSmoothResize = pSmooth
    
    Select bSndFont
        Case 0
            $ifdef FontSnd0
                bFontSpace = FontSndSpace0
                bFontPixels = FontSndPixels0
                bFontOffset = FontSndOffset0
                dwFontAddress = AddressOf(TableFont0)
            $endif
        Case 1
            $ifdef FontSnd1
                bFontSpace = FontSndSpace1
                bFontPixels = FontSndPixels1
                bFontOffset = FontSndOffset1
                dwFontAddress = AddressOf(TableFont1)
            $endif
        Case 2
            $ifdef FontSnd2
                bFontSpace = FontSndSpace2
                bFontPixels = FontSndPixels2
                bFontOffset = FontSndOffset2
                dwFontAddress = AddressOf(TableFont2)
            $endif
       Case 3
            $ifdef FontSnd3
                bFontSpace = FontSndSpace3
                bFontPixels = FontSndPixels3
                bFontOffset = FontSndOffset3
                dwFontAddress = AddressOf(TableFont3)
            $endif
        Case 4
            $ifdef FontSnd4
                bFontSpace = FontSndSpace4
                bFontPixels = FontSndPixels4
                bFontOffset = FontSndOffset4
                dwFontAddress = AddressOf(TableFont4)
            $endif
        Case 5
            $ifdef FontSnd5
                bFontSpace = FontSndSpace5
                bFontPixels = FontSndPixels5
                bFontOffset = FontSndOffset5
                dwFontAddress = AddressOf(TableFont5)
            $endif  
        Case 6
            $ifdef FontSnd6
                bFontSpace = FontSndSpace6
                bFontPixels = FontSndPixels6
                bFontOffset = FontSndOffset6
                dwFontAddress = AddressOf(TableFont6)
            $endif
        Case 7
            $ifdef FontSnd7
                bFontSpace = FontSndSpace7
                bFontPixels = FontSndPixels7
                bFontOffset = FontSndOffset7
                dwFontAddress = AddressOf(TableFont7)
            $endif
        Case 8
            $ifdef FontSnd8
                bFontSpace = FontSndSpace8
                bFontPixels = FontSndPixels8
                bFontOffset = FontSndOffset8
                dwFontAddress = AddressOf(TableFont8)
            $endif
        Case 9
            $ifdef FontSnd9
                bFontSpace = FontSndSpace9
                bFontPixels = FontSndPixels9
                bFontOffset = FontSndOffset9
                dwFontAddress = AddressOf(TableFont9)
            $endif
        Case 10
            $ifdef FontSnd10
                bFontSpace = FontSndSpace10
                bFontPixels = FontSndPixels10
                bFontOffset = FontSndOffset10
                dwFontAddress = AddressOf(TableFont10)
            $endif
        Case 11
            $ifdef FontSnd11
                bFontSpace = FontSndSpace11
                bFontPixels = FontSndPixels11
                bFontOffset = FontSndOffset11
                dwFontAddress = AddressOf(TableFont11)
            $endif
        Case 12
            $ifdef FontSnd12
                bFontSpace = FontSndSpace12
                bFontPixels = FontSndPixels12
                bFontOffset = FontSndOffset12
                dwFontAddress = AddressOf(TableFont12)
            $endif
        Case 13
            $ifdef FontSnd13
                bFontSpace = FontSndSpace13
                bFontPixels = FontSndPixels13
                bFontOffset = FontSndOffset13
                dwFontAddress = AddressOf(TableFont13)
            $endif
        Case 14
            $ifdef FontSnd14
                bFontSpace = FontSndSpace14
                bFontPixels = FontSndPixels14
                bFontOffset = FontSndOffset14
                dwFontAddress = AddressOf(TableFont14)
            $endif
        Case 15
            $ifdef FontSnd15
                bFontSpace = FontSndSpace15
                bFontPixels = FontSndPixels15
                bFontOffset = FontSndOffset15
                dwFontAddress = AddressOf(TableFont15)
            $endif
        Case 16
            $ifdef FontSnd16
                bFontSpace = FontSndSpace16
                bFontPixels = FontSndPixels16
                bFontOffset = FontSndOffset16
                dwFontAddress = AddressOf(TableFont16)
            $endif
        Case 17
            $ifdef FontSnd17
                bFontSpace = FontSndSpace17
                bFontPixels = FontSndPixels17
                bFontOffset = FontSndOffset17
                dwFontAddress = AddressOf(TableFont17)
            $endif
        Case 18
            $ifdef FontSnd18
                bFontSpace = FontSndSpace18
                bFontPixels = FontSndPixels18
                bFontOffset = FontSndOffset18
                dwFontAddress = AddressOf(TableFont18)
            $endif
        Case 19
            $ifdef FontSnd19
                bFontSpace = FontSndSpace19
                bFontPixels = FontSndPixels19
                bFontOffset = FontSndOffset19
                dwFontAddress = AddressOf(TableFont19)
            $endif
        Case 20
            $ifdef FontSnd20
                bFontSpace = FontSndSpace20
                bFontPixels = FontSndPixels20
                bFontOffset = FontSndOffset20
                dwFontAddress = AddressOf(TableFont20)
            $endif
        Case 21
            $ifdef FontSnd21
                bFontSpace = FontSndSpace21
                bFontPixels = FontSndPixels21
                bFontOffset = FontSndOffset21
                dwFontAddress = AddressOf(TableFont21)
            $endif
        Case 22
            $ifdef FontSnd22
                bFontSpace = FontSndSpace22
                bFontPixels = FontSndPixels22
                bFontOffset = FontSndOffset22
                dwFontAddress = AddressOf(TableFont22)
            $endif
        Case 23
            $ifdef FontSnd23
                bFontSpace = FontSndSpace23
                bFontPixels = FontSndPixels23
                bFontOffset = FontSndOffset23
                dwFontAddress = AddressOf(TableFont23)
            $endif
        Case 24
            $ifdef FontSnd24
                bFontSpace = FontSndSpace24
                bFontPixels = FontSndPixels24
                bFontOffset = FontSndOffset24
                dwFontAddress = AddressOf(TableFont24)
            $endif
        Case 25
            $ifdef FontSnd25
                bFontSpace = FontSndSpace25
                bFontPixels = FontSndPixels25
                bFontOffset = FontSndOffset25
                dwFontAddress = AddressOf(TableFont25)
            $endif
        Case 26
            $ifdef FontSnd26
                bFontSpace = FontSndSpace26
                bFontPixels = FontSndPixels26
                bFontOffset = FontSndOffset26
                dwFontAddress = AddressOf(TableFont26)
            $endif
        Case 27
            $ifdef FontSnd27
                bFontSpace = FontSndSpace27
                bFontPixels = FontSndPixels27
                bFontOffset = FontSndOffset27
                dwFontAddress = AddressOf(TableFont27)
            $endif
        Case 28
            $ifdef FontSnd28
                bFontSpace = FontSndSpace28
                bFontPixels = FontSndPixels28
                bFontOffset = FontSndOffset28
                dwFontAddress = AddressOf(TableFont28)
            $endif
        Case 29
            $ifdef FontSnd29
                bFontSpace = FontSndSpace29
                bFontPixels = FontSndPixels29
                bFontOffset = FontSndOffset29
                dwFontAddress = AddressOf(TableFont29)
            $endif
        Case 30
            $ifdef FontSnd30
                bFontSpace = FontSndSpace30
                bFontPixels = FontSndPixels30
                bFontOffset = FontSndOffset30
                dwFontAddress = AddressOf(TableFont30)
            $endif
        Case 31
            $ifdef FontSnd31
                bFontSpace = FontSndSpace31
                bFontPixels = FontSndPixels31
                bFontOffset = FontSndOffset31
                dwFontAddress = AddressOf(TableFont31)
            $endif
        Case 32
            $ifdef FontSnd32
                bFontSpace = FontSndSpace32
                bFontPixels = FontSndPixels32
                bFontOffset = FontSndOffset32
                dwFontAddress = AddressOf(TableFont32)
            $endif
        Case 33
            $ifdef FontSnd33
                bFontSpace = FontSndSpace33
                bFontPixels = FontSndPixels33
                bFontOffset = FontSndOffset33
                dwFontAddress = AddressOf(TableFont33)
            $endif
        Case 34
            $ifdef FontSnd34
                bFontSpace = FontSndSpace34
                bFontPixels = FontSndPixels34
                bFontOffset = FontSndOffset34
                dwFontAddress = AddressOf(TableFont34)
            $endif
        Case 35
            $ifdef FontSnd35
                bFontSpace = FontSndSpace35
                bFontPixels = FontSndPixels35
                bFontOffset = FontSndOffset35
                dwFontAddress = AddressOf(TableFont35)
            $endif
        Case 36
            $ifdef FontSnd36
                bFontSpace = FontSndSpace36
                bFontPixels = FontSndPixels36
                bFontOffset = FontSndOffset36
                dwFontAddress = AddressOf(TableFont36)
            $endif
        Case 37
            $ifdef FontSnd37
                bFontSpace = FontSndSpace37
                bFontPixels = FontSndPixels37
                bFontOffset = FontSndOffset37
                dwFontAddress = AddressOf(TableFont37)
            $endif
        Case 38
            $ifdef FontSnd38
                bFontSpace = FontSndSpace38
                bFontPixels = FontSndPixels38
                bFontOffset = FontSndOffset38
                dwFontAddress = AddressOf(TableFont38)
            $endif
        Case 39
            $ifdef FontSnd39
                bFontSpace = FontSndSpace39
                bFontPixels = FontSndPixels39
                bFontOffset = FontSndOffset39
                dwFontAddress = AddressOf(TableFont39)
            $endif
        Case 40
            $ifdef FontSnd40
                bFontSpace = FontSndSpace40
                bFontPixels = FontSndPixels40
                bFontOffset = FontSndOffset40
                dwFontAddress = AddressOf(TableFont40)
            $endif
        Case 41
            $ifdef FontSnd41
                bFontSpace = FontSndSpace41
                bFontPixels = FontSndPixels41
                bFontOffset = FontSndOffset41
                dwFontAddress = AddressOf(TableFont41)
            $endif
        Case 42
            $ifdef FontSnd42
                bFontSpace = FontSndSpace42
                bFontPixels = FontSndPixels42
                bFontOffset = FontSndOffset42
                dwFontAddress = AddressOf(TableFont42)
            $endif
        Case 43
            $ifdef FontSnd43
                bFontSpace = FontSndSpace43
                bFontPixels = FontSndPixels43
                bFontOffset = FontSndOffset43
                dwFontAddress = AddressOf(TableFont43)
            $endif
        Case 44
            $ifdef FontSnd44
                bFontSpace = FontSndSpace44
                bFontPixels = FontSndPixels44
                bFontOffset = FontSndOffset44
                dwFontAddress = AddressOf(TableFont44)
            $endif
        Case 45
            $ifdef FontSnd45
                bFontSpace = FontSndSpace45
                bFontPixels = FontSndPixels45
                bFontOffset = FontSndOffset45
                dwFontAddress = AddressOf(TableFont45)
            $endif
        Case 46
            $ifdef FontSnd46
                bFontSpace = FontSndSpace46
                bFontPixels = FontSndPixels46
                bFontOffset = FontSndOffset46
                dwFontAddress = AddressOf(TableFont46)
            $endif
        Case 47
            $ifdef FontSnd47
                bFontSpace = FontSndSpace47
                bFontPixels = FontSndPixels47
                bFontOffset = FontSndOffset47
                dwFontAddress = AddressOf(TableFont47)
            $endif
        Case 48
            $ifdef FontSnd48
                bFontSpace = FontSndSpace48
                bFontPixels = FontSndPixels48
                bFontOffset = FontSndOffset48
                dwFontAddress = AddressOf(TableFont48)
            $endif
        Case 49
            $ifdef FontSnd49
                bFontSpace = FontSndSpace49
                bFontPixels = FontSndPixels49
                bFontOffset = FontSndOffset49
                dwFontAddress = AddressOf(TableFont49)
            $endif
        Case 50
            $ifdef FontSnd50
                bFontSpace = FontSndSpace50
                bFontPixels = FontSndPixels50
                bFontOffset = FontSndOffset50
                dwFontAddress = AddressOf(TableFont50)
            $endif
        Case 51
            $ifdef FontSnd51
                bFontSpace = FontSndSpace51
                bFontPixels = FontSndPixels51
                bFontOffset = FontSndOffset51
                dwFontAddress = AddressOf(TableFont51)
            $endif
        Case 52
            $ifdef FontSnd52
                bFontSpace = FontSndSpace52
                bFontPixels = FontSndPixels52
                bFontOffset = FontSndOffset52
                dwFontAddress = AddressOf(TableFont52)
            $endif
        Case 53
            $ifdef FontSnd53
                bFontSpace = FontSndSpace53
                bFontPixels = FontSndPixels53
                bFontOffset = FontSndOffset53
                dwFontAddress = AddressOf(TableFont53)
            $endif
        Case 54
            $ifdef FontSnd54
                bFontSpace = FontSndSpace54
                bFontPixels = FontSndPixels54
                bFontOffset = FontSndOffset54
                dwFontAddress = AddressOf(TableFont54)
            $endif
        Case 55
            $ifdef FontSnd55
                bFontSpace = FontSndSpace55
                bFontPixels = FontSndPixels55
                bFontOffset = FontSndOffset55
                dwFontAddress = AddressOf(TableFont55)
            $endif
        Case 56
            $ifdef FontSnd56
                bFontSpace = FontSndSpace56
                bFontPixels = FontSndPixels56
                bFontOffset = FontSndOffset56
                dwFontAddress = AddressOf(TableFont56)
            $endif
        Case 57
            $ifdef FontSnd57
                bFontSpace = FontSndSpace57
                bFontPixels = FontSndPixels57
                bFontOffset = FontSndOffset57
                dwFontAddress = AddressOf(TableFont57)
            $endif
        Case 58
            $ifdef FontSnd58
                bFontSpace = FontSndSpace58
                bFontPixels = FontSndPixels58
                bFontOffset = FontSndOffset58
                dwFontAddress = AddressOf(TableFont58)
            $endif
        Case 59
            $ifdef FontSnd59
                bFontSpace = FontSndSpace59
                bFontPixels = FontSndPixels59
                bFontOffset = FontSndOffset59
                dwFontAddress = AddressOf(TableFont59)
            $endif
        Case 60
            $ifdef FontSnd60
                bFontSpace = FontSndSpace60
                bFontPixels = FontSndPixels60
                bFontOffset = FontSndOffset60
                dwFontAddress = AddressOf(TableFont60)
            $endif
        Case 61
            $ifdef FontSnd61
                bFontSpace = FontSndSpace61
                bFontPixels = FontSndPixels61
                bFontOffset = FontSndOffset61
                dwFontAddress = AddressOf(TableFont61)
            $endif
        Case 62
            $ifdef FontSnd62
                bFontSpace = FontSndSpace62
                bFontPixels = FontSndPixels62
                bFontOffset = FontSndOffset62
                dwFontAddress = AddressOf(TableFont62)
            $endif
        Case 63
            $ifdef FontSnd63
                bFontSpace = FontSndSpace63
                bFontPixels = FontSndPixels63
                bFontOffset = FontSndOffset63
                dwFontAddress = AddressOf(TableFont63)
            $endif
        Case 64
            $ifdef FontSnd64
                bFontSpace = FontSndSpace64
                bFontPixels = FontSndPixels64
                bFontOffset = FontSndOffset64
                dwFontAddress = AddressOf(TableFont64)
            $endif
        Case 65
            $ifdef FontSnd65
                bFontSpace = FontSndSpace65
                bFontPixels = FontSndPixels65
                bFontOffset = FontSndOffset65
                dwFontAddress = AddressOf(TableFont65)
            $endif
        Case 66
            $ifdef FontSnd66
                bFontSpace = FontSndSpace66
                bFontPixels = FontSndPixels66
                bFontOffset = FontSndOffset66
                dwFontAddress = AddressOf(TableFont66)
            $endif
        Case 67
            $ifdef FontSnd67
                bFontSpace = FontSndSpace67
                bFontPixels = FontSndPixels67
                bFontOffset = FontSndOffset67
                dwFontAddress = AddressOf(TableFont67)
            $endif
        Case 68
            $ifdef FontSnd68
                bFontSpace = FontSndSpace68
                bFontPixels = FontSndPixels68
                bFontOffset = FontSndOffset68
                dwFontAddress = AddressOf(TableFont68)
            $endif
        Case 69
            $ifdef FontSnd69
                bFontSpace = FontSndSpace69
                bFontPixels = FontSndPixels69
                bFontOffset = FontSndOffset69
                dwFontAddress = AddressOf(TableFont69)
            $endif
        Case 70
            $ifdef FontSnd70
                bFontSpace = FontSndSpace70
                bFontPixels = FontSndPixels70
                bFontOffset = FontSndOffset70
                dwFontAddress = AddressOf(TableFont70)
            $endif
        Case 71
            $ifdef FontSnd71
                bFontSpace = FontSndSpace71
                bFontPixels = FontSndPixels71
                bFontOffset = FontSndOffset71
                dwFontAddress = AddressOf(TableFont71)
            $endif
        Case 72
            $ifdef FontSnd72
                bFontSpace = FontSndSpace72
                bFontPixels = FontSndPixels72
                bFontOffset = FontSndOffset72
                dwFontAddress = AddressOf(TableFont72)
            $endif
        Case 73
            $ifdef FontSnd73
                bFontSpace = FontSndSpace73
                bFontPixels = FontSndPixels73
                bFontOffset = FontSndOffset73
                dwFontAddress = AddressOf(TableFont73)
            $endif
        Case 74
            $ifdef FontSnd74
                bFontSpace = FontSndSpace74
                bFontPixels = FontSndPixels74
                bFontOffset = FontSndOffset74
                dwFontAddress = AddressOf(TableFont74)
            $endif
        Case 75
            $ifdef FontSnd75
                bFontSpace = FontSndSpace75
                bFontPixels = FontSndPixels75
                bFontOffset = FontSndOffset75
                dwFontAddress = AddressOf(TableFont75)
            $endif
        Case 76
            $ifdef FontSnd76
                bFontSpace = FontSndSpace76
                bFontPixels = FontSndPixels76
                bFontOffset = FontSndOffset76
                dwFontAddress = AddressOf(TableFont76)
            $endif
        Case 77
            $ifdef FontSnd77
                bFontSpace = FontSndSpace77
                bFontPixels = FontSndPixels77
                bFontOffset = FontSndOffset77
                dwFontAddress = AddressOf(TableFont77)
            $endif
        Case 78
            $ifdef FontSnd78
                bFontSpace = FontSndSpace78
                bFontPixels = FontSndPixels78
                bFontOffset = FontSndOffset78
                dwFontAddress = AddressOf(TableFont78)
            $endif
        Case 79
            $ifdef FontSnd79
                bFontSpace = FontSndSpace79
                bFontPixels = FontSndPixels79
                bFontOffset = FontSndOffset79
                dwFontAddress = AddressOf(TableFont79)
            $endif
        Case 80
            $ifdef FontSnd80
                bFontSpace = FontSndSpace80
                bFontPixels = FontSndPixels80
                bFontOffset = FontSndOffset80
                dwFontAddress = AddressOf(TableFont80)
            $endif
        Case 81
            $ifdef FontSnd81
                bFontSpace = FontSndSpace81
                bFontPixels = FontSndPixels81
                bFontOffset = FontSndOffset81
                dwFontAddress = AddressOf(TableFont81)
            $endif
        Case 82
            $ifdef FontSnd82
                bFontSpace = FontSndSpace82
                bFontPixels = FontSndPixels82
                bFontOffset = FontSndOffset82
                dwFontAddress = AddressOf(TableFont82)
            $endif
        Case 83
            $ifdef FontSnd83
                bFontSpace = FontSndSpace83
                bFontPixels = FontSndPixels83
                bFontOffset = FontSndOffset83
                dwFontAddress = AddressOf(TableFont83)
            $endif
        Case 84
            $ifdef FontSnd84
                bFontSpace = FontSndSpace84
                bFontPixels = FontSndPixels84
                bFontOffset = FontSndOffset84
                dwFontAddress = AddressOf(TableFont84)
            $endif
        Case 85
            $ifdef FontSnd85
                bFontSpace = FontSndSpace85
                bFontPixels = FontSndPixels85
                bFontOffset = FontSndOffset85
                dwFontAddress = AddressOf(TableFont85)
            $endif
        Case 86
            $ifdef FontSnd86
                bFontSpace = FontSndSpace86
                bFontPixels = FontSndPixels86
                bFontOffset = FontSndOffset86
                dwFontAddress = AddressOf(TableFont86)
            $endif
        Case 87
            $ifdef FontSnd87
                bFontSpace = FontSndSpace87
                bFontPixels = FontSndPixels87
                bFontOffset = FontSndOffset87
                dwFontAddress = AddressOf(TableFont87)
            $endif
        Case 88
            $ifdef FontSnd88
                bFontSpace = FontSndSpace88
                bFontPixels = FontSndPixels88
                bFontOffset = FontSndOffset88
                dwFontAddress = AddressOf(TableFont88)
            $endif
        Case 89
            $ifdef FontSnd89
                bFontSpace = FontSndSpace89
                bFontPixels = FontSndPixels89
                bFontOffset = FontSndOffset89
                dwFontAddress = AddressOf(TableFont89)
            $endif
        Case 90
            $ifdef FontSnd90
                bFontSpace = FontSndSpace90
                bFontPixels = FontSndPixels90
                bFontOffset = FontSndOffset90
                dwFontAddress = AddressOf(TableFont90)
            $endif
        Case 91
            $ifdef FontSnd91
                bFontSpace = FontSndSpace91
                bFontPixels = FontSndPixels91
                bFontOffset = FontSndOffset91
                dwFontAddress = AddressOf(TableFont91)
            $endif
        Case 92
            $ifdef FontSnd92
                bFontSpace = FontSndSpace92
                bFontPixels = FontSndPixels92
                bFontOffset = FontSndOffset92
                dwFontAddress = AddressOf(TableFont92)
            $endif
        Case 93
            $ifdef FontSnd93
                bFontSpace = FontSndSpace93
                bFontPixels = FontSndPixels93
                bFontOffset = FontSndOffset93
                dwFontAddress = AddressOf(TableFont93)
            $endif
        Case 94
            $ifdef FontSnd94
                bFontSpace = FontSndSpace94
                bFontPixels = FontSndPixels94
                bFontOffset = FontSndOffset94
                dwFontAddress = AddressOf(TableFont94)
            $endif
        Case 95
            $ifdef FontSnd95
                bFontSpace = FontSndSpace95
                bFontPixels = FontSndPixels95
                bFontOffset = FontSndOffset95
                dwFontAddress = AddressOf(TableFont95)
            $endif
        Case 96
            $ifdef FontSnd96
                bFontSpace = FontSndSpace96
                bFontPixels = FontSndPixels96
                bFontOffset = FontSndOffset96
                dwFontAddress = AddressOf(TableFont96)
            $endif
        Case 97
            $ifdef FontSnd97
                bFontSpace = FontSndSpace97
                bFontPixels = FontSndPixels97
                bFontOffset = FontSndOffset97
                dwFontAddress = AddressOf(TableFont97)
            $endif
        Case 98
            $ifdef FontSnd98
                bFontSpace = FontSndSpace98
                bFontPixels = FontSndPixels98
                bFontOffset = FontSndOffset98
                dwFontAddress = AddressOf(TableFont98)
            $endif
        Case 99
            $ifdef FontSnd99
                bFontSpace = FontSndSpace99
                bFontPixels = FontSndPixels99
                bFontOffset = FontSndOffset99
                dwFontAddress = AddressOf(TableFont99)
            $endif
        Case 100
            $ifdef FontSnd100
                bFontSpace = FontSndSpace100
                bFontPixels = FontSndPixels100
                bFontOffset = FontSndOffset100
                dwFontAddress = AddressOf(TableFont100)
            $endif
        Case 101
            $ifdef FontSnd101
                bFontSpace = FontSndSpace101
                bFontPixels = FontSndPixels101
                bFontOffset = FontSndOffset101
                dwFontAddress = AddressOf(TableFont101)
            $endif
        Case 102
            $ifdef FontSnd102
                bFontSpace = FontSndSpace102
                bFontPixels = FontSndPixels102
                bFontOffset = FontSndOffset102
                dwFontAddress = AddressOf(TableFont102)
            $endif
        Case 103
            $ifdef FontSnd103
                bFontSpace = FontSndSpace103
                bFontPixels = FontSndPixels103
                bFontOffset = FontSndOffset103
                dwFontAddress = AddressOf(TableFont103)
            $endif
        Case 104
            $ifdef FontSnd104
                bFontSpace = FontSndSpace104
                bFontPixels = FontSndPixels104
                bFontOffset = FontSndOffset104
                dwFontAddress = AddressOf(TableFont104)
            $endif
        Case 105
            $ifdef FontSnd105
                bFontSpace = FontSndSpace105
                bFontPixels = FontSndPixels105
                bFontOffset = FontSndOffset105
                dwFontAddress = AddressOf(TableFont105)
            $endif
        Case 106
            $ifdef FontSnd106
                bFontSpace = FontSndSpace106
                bFontPixels = FontSndPixels106
                bFontOffset = FontSndOffset106
                dwFontAddress = AddressOf(TableFont106)
            $endif
        Case 107
            $ifdef FontSnd107
                bFontSpace = FontSndSpace107
                bFontPixels = FontSndPixels107
                bFontOffset = FontSndOffset107
                dwFontAddress = AddressOf(TableFont107)
            $endif
        Case 108
            $ifdef FontSnd108
                bFontSpace = FontSndSpace108
                bFontPixels = FontSndPixels108
                bFontOffset = FontSndOffset108
                dwFontAddress = AddressOf(TableFont108)
            $endif
        Case 109
            $ifdef FontSnd109
                bFontSpace = FontSndSpace109
                bFontPixels = FontSndPixels109
                bFontOffset = FontSndOffset109
                dwFontAddress = AddressOf(TableFont109)
            $endif
        Case 110
            $ifdef FontSnd110
                bFontSpace = FontSndSpace110
                bFontPixels = FontSndPixels110
                bFontOffset = FontSndOffset110
                dwFontAddress = AddressOf(TableFont110)
            $endif
        Case 111
            $ifdef FontSnd111
                bFontSpace = FontSndSpace111
                bFontPixels = FontSndPixels111
                bFontOffset = FontSndOffset111
                dwFontAddress = AddressOf(TableFont111)
            $endif
        Case 112
            $ifdef FontSnd112
                bFontSpace = FontSndSpace112
                bFontPixels = FontSndPixels112
                bFontOffset = FontSndOffset112
                dwFontAddress = AddressOf(TableFont112)
            $endif
        Case 113
            $ifdef FontSnd113
                bFontSpace = FontSndSpace113
                bFontPixels = FontSndPixels113
                bFontOffset = FontSndOffset113
                dwFontAddress = AddressOf(TableFont113)
            $endif
        Case 114
            $ifdef FontSnd114
                bFontSpace = FontSndSpace114
                bFontPixels = FontSndPixels114
                bFontOffset = FontSndOffset114
                dwFontAddress = AddressOf(TableFont114)
            $endif
        Case 115
            $ifdef FontSnd115
                bFontSpace = FontSndSpace115
                bFontPixels = FontSndPixels115
                bFontOffset = FontSndOffset115
                dwFontAddress = AddressOf(TableFont115)
            $endif
        Case 116
            $ifdef FontSnd116
                bFontSpace = FontSndSpace116
                bFontPixels = FontSndPixels116
                bFontOffset = FontSndOffset116
                dwFontAddress = AddressOf(TableFont116)
            $endif
        Case 117
            $ifdef FontSnd117
                bFontSpace = FontSndSpace117
                bFontPixels = FontSndPixels117
                bFontOffset = FontSndOffset117
                dwFontAddress = AddressOf(TableFont117)
            $endif
        Case 118
            $ifdef FontSnd118
                bFontSpace = FontSndSpace118
                bFontPixels = FontSndPixels118
                bFontOffset = FontSndOffset118
                dwFontAddress = AddressOf(TableFont118)
            $endif
        Case 119
            $ifdef FontSnd119
                bFontSpace = FontSndSpace119
                bFontPixels = FontSndPixels119
                bFontOffset = FontSndOffset119
                dwFontAddress = AddressOf(TableFont119)
            $endif
        Case 120
            $ifdef FontSnd120
                bFontSpace = FontSndSpace120
                bFontPixels = FontSndPixels120
                bFontOffset = FontSndOffset120
                dwFontAddress = AddressOf(TableFont120)
            $endif
        Case 121
            $ifdef FontSnd121
                bFontSpace = FontSndSpace121
                bFontPixels = FontSndPixels121
                bFontOffset = FontSndOffset121
                dwFontAddress = AddressOf(TableFont121)
            $endif
        Case 122
            $ifdef FontSnd122
                bFontSpace = FontSndSpace122
                bFontPixels = FontSndPixels122
                bFontOffset = FontSndOffset122
                dwFontAddress = AddressOf(TableFont122)
            $endif
        Case 123
            $ifdef FontSnd123
                bFontSpace = FontSndSpace123
                bFontPixels = FontSndPixels123
                bFontOffset = FontSndOffset123
                dwFontAddress = AddressOf(TableFont123)
            $endif
        Case 124
            $ifdef FontSnd124
                bFontSpace = FontSndSpace124
                bFontPixels = FontSndPixels124
                bFontOffset = FontSndOffset124
                dwFontAddress = AddressOf(TableFont124)
            $endif
        Case 125
            $ifdef FontSnd125
                bFontSpace = FontSndSpace125
                bFontPixels = FontSndPixels125
                bFontOffset = FontSndOffset125
                dwFontAddress = AddressOf(TableFont125)
            $endif
        Case 126
            $ifdef FontSnd126
                bFontSpace = FontSndSpace126
                bFontPixels = FontSndPixels126
                bFontOffset = FontSndOffset126
                dwFontAddress = AddressOf(TableFont126)
            $endif
        Case 127
            $ifdef FontSnd127
                bFontSpace = FontSndSpace127
                bFontPixels = FontSndPixels127
                bFontOffset = FontSndOffset127
                dwFontAddress = AddressOf(TableFont127)
            $endif
        Case 128
            $ifdef FontSnd128
                bFontSpace = FontSndSpace128
                bFontPixels = FontSndPixels128
                bFontOffset = FontSndOffset128
                dwFontAddress = AddressOf(TableFont128)
            $endif
        Case 129
            $ifdef FontSnd129
                bFontSpace = FontSndSpace129
                bFontPixels = FontSndPixels129
                bFontOffset = FontSndOffset129
                dwFontAddress = AddressOf(TableFont129)
            $endif
        Case 130
            $ifdef FontSnd130
                bFontSpace = FontSndSpace130
                bFontPixels = FontSndPixels130
                bFontOffset = FontSndOffset130
                dwFontAddress = AddressOf(TableFont130)
            $endif
        Case 131
            $ifdef FontSnd131
                bFontSpace = FontSndSpace131
                bFontPixels = FontSndPixels131
                bFontOffset = FontSndOffset131
                dwFontAddress = AddressOf(TableFont131)
            $endif
        Case 132
            $ifdef FontSnd132
                bFontSpace = FontSndSpace132
                bFontPixels = FontSndPixels132
                bFontOffset = FontSndOffset132
                dwFontAddress = AddressOf(TableFont132)
            $endif
        Case 133
            $ifdef FontSnd133
                bFontSpace = FontSndSpace133
                bFontPixels = FontSndPixels133
                bFontOffset = FontSndOffset133
                dwFontAddress = AddressOf(TableFont133)
            $endif
        Case 134
            $ifdef FontSnd134
                bFontSpace = FontSndSpace134
                bFontPixels = FontSndPixels134
                bFontOffset = FontSndOffset134
                dwFontAddress = AddressOf(TableFont134)
            $endif
        Case 135
            $ifdef FontSnd135
                bFontSpace = FontSndSpace135
                bFontPixels = FontSndPixels135
                bFontOffset = FontSndOffset135
                dwFontAddress = AddressOf(TableFont135)
            $endif
        Case 136
            $ifdef FontSnd136
                bFontSpace = FontSndSpace136
                bFontPixels = FontSndPixels136
                bFontOffset = FontSndOffset136
                dwFontAddress = AddressOf(TableFont136)
            $endif
        Case 137
            $ifdef FontSnd137
                bFontSpace = FontSndSpace137
                bFontPixels = FontSndPixels137
                bFontOffset = FontSndOffset137
                dwFontAddress = AddressOf(TableFont137)
            $endif
        Case 138
            $ifdef FontSnd138
                bFontSpace = FontSndSpace138
                bFontPixels = FontSndPixels138
                bFontOffset = FontSndOffset138
                dwFontAddress = AddressOf(TableFont138)
            $endif
        Case 139
            $ifdef FontSnd139
                bFontSpace = FontSndSpace139
                bFontPixels = FontSndPixels139
                bFontOffset = FontSndOffset139
                dwFontAddress = AddressOf(TableFont139)
            $endif
        Case 140
            $ifdef FontSnd140
                bFontSpace = FontSndSpace140
                bFontPixels = FontSndPixels140
                bFontOffset = FontSndOffset140
                dwFontAddress = AddressOf(TableFont140)
            $endif
        Case 141
            $ifdef FontSnd141
                bFontSpace = FontSndSpace141
                bFontPixels = FontSndPixels141
                bFontOffset = FontSndOffset141
                dwFontAddress = AddressOf(TableFont141)
            $endif
        Case 142
            $ifdef FontSnd142
                bFontSpace = FontSndSpace142
                bFontPixels = FontSndPixels142
                bFontOffset = FontSndOffset142
                dwFontAddress = AddressOf(TableFont142)
            $endif
        Case 143
            $ifdef FontSnd143
                bFontSpace = FontSndSpace143
                bFontPixels = FontSndPixels143
                bFontOffset = FontSndOffset143
                dwFontAddress = AddressOf(TableFont143)
            $endif
        Case 144
            $ifdef FontSnd144
                bFontSpace = FontSndSpace144
                bFontPixels = FontSndPixels144
                bFontOffset = FontSndOffset144
                dwFontAddress = AddressOf(TableFont144)
            $endif
        Case 145
            $ifdef FontSnd145
                bFontSpace = FontSndSpace145
                bFontPixels = FontSndPixels145
                bFontOffset = FontSndOffset145
                dwFontAddress = AddressOf(TableFont145)
            $endif
        Case 146
            $ifdef FontSnd146
                bFontSpace = FontSndSpace146
                bFontPixels = FontSndPixels146
                bFontOffset = FontSndOffset146
                dwFontAddress = AddressOf(TableFont146)
            $endif
        Case 147
            $ifdef FontSnd147
                bFontSpace = FontSndSpace147
                bFontPixels = FontSndPixels147
                bFontOffset = FontSndOffset147
                dwFontAddress = AddressOf(TableFont147)
            $endif
        Case 148
            $ifdef FontSnd148
                bFontSpace = FontSndSpace148
                bFontPixels = FontSndPixels148
                bFontOffset = FontSndOffset148
                dwFontAddress = AddressOf(TableFont148)
            $endif
        Case 149
            $ifdef FontSnd149
                bFontSpace = FontSndSpace149
                bFontPixels = FontSndPixels149
                bFontOffset = FontSndOffset149
                dwFontAddress = AddressOf(TableFont149)
            $endif
        Case 150
            $ifdef FontSnd150
                bFontSpace = FontSndSpace150
                bFontPixels = FontSndPixels150
                bFontOffset = FontSndOffset150
                dwFontAddress = AddressOf(TableFont150)
            $endif
        Case 151
            $ifdef FontSnd151
                bFontSpace = FontSndSpace151
                bFontPixels = FontSndPixels151
                bFontOffset = FontSndOffset151
                dwFontAddress = AddressOf(TableFont151)
            $endif
        Case 152
            $ifdef FontSnd152
                bFontSpace = FontSndSpace152
                bFontPixels = FontSndPixels152
                bFontOffset = FontSndOffset152
                dwFontAddress = AddressOf(TableFont152)
            $endif
        Case 153
            $ifdef FontSnd153
                bFontSpace = FontSndSpace153
                bFontPixels = FontSndPixels153
                bFontOffset = FontSndOffset153
                dwFontAddress = AddressOf(TableFont153)
            $endif
        Case 154
            $ifdef FontSnd154
                bFontSpace = FontSndSpace154
                bFontPixels = FontSndPixels154
                bFontOffset = FontSndOffset154
                dwFontAddress = AddressOf(TableFont154)
            $endif
        Case 155
            $ifdef FontSnd155
                bFontSpace = FontSndSpace155
                bFontPixels = FontSndPixels155
                bFontOffset = FontSndOffset155
                dwFontAddress = AddressOf(TableFont155)
            $endif
        Case 156
            $ifdef FontSnd156
                bFontSpace = FontSndSpace156
                bFontPixels = FontSndPixels156
                bFontOffset = FontSndOffset156
                dwFontAddress = AddressOf(TableFont156)
            $endif
        Case 157
            $ifdef FontSnd157
                bFontSpace = FontSndSpace157
                bFontPixels = FontSndPixels157
                bFontOffset = FontSndOffset157
                dwFontAddress = AddressOf(TableFont157)
            $endif
        Case 158
            $ifdef FontSnd158
                bFontSpace = FontSndSpace158
                bFontPixels = FontSndPixels158
                bFontOffset = FontSndOffset158
                dwFontAddress = AddressOf(TableFont158)
            $endif
        Case 159
            $ifdef FontSnd159
                bFontSpace = FontSndSpace159
                bFontPixels = FontSndPixels159
                bFontOffset = FontSndOffset159
                dwFontAddress = AddressOf(TableFont159)
            $endif
        Case 160
            $ifdef FontSnd160
                bFontSpace = FontSndSpace160
                bFontPixels = FontSndPixels160
                bFontOffset = FontSndOffset160
                dwFontAddress = AddressOf(TableFont160)
            $endif
        Case 161
            $ifdef FontSnd161
                bFontSpace = FontSndSpace161
                bFontPixels = FontSndPixels161
                bFontOffset = FontSndOffset161
                dwFontAddress = AddressOf(TableFont161)
            $endif
        Case 162
            $ifdef FontSnd162
                bFontSpace = FontSndSpace162
                bFontPixels = FontSndPixels162
                bFontOffset = FontSndOffset162
                dwFontAddress = AddressOf(TableFont162)
            $endif
        Case 163
            $ifdef FontSnd163
                bFontSpace = FontSndSpace163
                bFontPixels = FontSndPixels163
                bFontOffset = FontSndOffset163
                dwFontAddress = AddressOf(TableFont163)
            $endif
        Case 164
            $ifdef FontSnd164
                bFontSpace = FontSndSpace164
                bFontPixels = FontSndPixels164
                bFontOffset = FontSndOffset164
                dwFontAddress = AddressOf(TableFont164)
            $endif
        Case 165
            $ifdef FontSnd165
                bFontSpace = FontSndSpace165
                bFontPixels = FontSndPixels165
                bFontOffset = FontSndOffset165
                dwFontAddress = AddressOf(TableFont165)
            $endif
        Case 166
            $ifdef FontSnd166
                bFontSpace = FontSndSpace166
                bFontPixels = FontSndPixels166
                bFontOffset = FontSndOffset166
                dwFontAddress = AddressOf(TableFont166)
            $endif
        Case 167
            $ifdef FontSnd167
                bFontSpace = FontSndSpace167
                bFontPixels = FontSndPixels167
                bFontOffset = FontSndOffset167
                dwFontAddress = AddressOf(TableFont167)
            $endif
        Case 168
            $ifdef FontSnd168
                bFontSpace = FontSndSpace168
                bFontPixels = FontSndPixels168
                bFontOffset = FontSndOffset168
                dwFontAddress = AddressOf(TableFont168)
            $endif
        Case 169
            $ifdef FontSnd169
                bFontSpace = FontSndSpace169
                bFontPixels = FontSndPixels169
                bFontOffset = FontSndOffset169
                dwFontAddress = AddressOf(TableFont169)
            $endif
        Case 170
            $ifdef FontSnd170
                bFontSpace = FontSndSpace170
                bFontPixels = FontSndPixels170
                bFontOffset = FontSndOffset170
                dwFontAddress = AddressOf(TableFont170)
            $endif
        Case 171
            $ifdef FontSnd171
                bFontSpace = FontSndSpace171
                bFontPixels = FontSndPixels171
                bFontOffset = FontSndOffset171
                dwFontAddress = AddressOf(TableFont171)
            $endif
        Case 172
            $ifdef FontSnd172
                bFontSpace = FontSndSpace172
                bFontPixels = FontSndPixels172
                bFontOffset = FontSndOffset172
                dwFontAddress = AddressOf(TableFont172)
            $endif
        Case 173
            $ifdef FontSnd173
                bFontSpace = FontSndSpace173
                bFontPixels = FontSndPixels173
                bFontOffset = FontSndOffset173
                dwFontAddress = AddressOf(TableFont173)
            $endif
        Case 174
            $ifdef FontSnd174
                bFontSpace = FontSndSpace174
                bFontPixels = FontSndPixels174
                bFontOffset = FontSndOffset174
                dwFontAddress = AddressOf(TableFont174)
            $endif
        Case 175
            $ifdef FontSnd175
                bFontSpace = FontSndSpace175
                bFontPixels = FontSndPixels175
                bFontOffset = FontSndOffset175
                dwFontAddress = AddressOf(TableFont175)
            $endif
        Case 176
            $ifdef FontSnd176
                bFontSpace = FontSndSpace176
                bFontPixels = FontSndPixels176
                bFontOffset = FontSndOffset176
                dwFontAddress = AddressOf(TableFont176)
            $endif
        Case 177
            $ifdef FontSnd177
                bFontSpace = FontSndSpace177
                bFontPixels = FontSndPixels177
                bFontOffset = FontSndOffset177
                dwFontAddress = AddressOf(TableFont177)
            $endif
        Case 178
            $ifdef FontSnd178
                bFontSpace = FontSndSpace178
                bFontPixels = FontSndPixels178
                bFontOffset = FontSndOffset178
                dwFontAddress = AddressOf(TableFont178)
            $endif
        Case 179
            $ifdef FontSnd179
                bFontSpace = FontSndSpace179
                bFontPixels = FontSndPixels179
                bFontOffset = FontSndOffset179
                dwFontAddress = AddressOf(TableFont179)
            $endif
        Case 180
            $ifdef FontSnd180
                bFontSpace = FontSndSpace180
                bFontPixels = FontSndPixels180
                bFontOffset = FontSndOffset180
                dwFontAddress = AddressOf(TableFont180)
            $endif
        Case 181
            $ifdef FontSnd181
                bFontSpace = FontSndSpace181
                bFontPixels = FontSndPixels181
                bFontOffset = FontSndOffset181
                dwFontAddress = AddressOf(TableFont181)
            $endif
        Case 182
            $ifdef FontSnd182
                bFontSpace = FontSndSpace182
                bFontPixels = FontSndPixels182
                bFontOffset = FontSndOffset182
                dwFontAddress = AddressOf(TableFont182)
            $endif
        Case 183
            $ifdef FontSnd183
                bFontSpace = FontSndSpace183
                bFontPixels = FontSndPixels183
                bFontOffset = FontSndOffset183
                dwFontAddress = AddressOf(TableFont183)
            $endif
        Case 184
            $ifdef FontSnd184
                bFontSpace = FontSndSpace184
                bFontPixels = FontSndPixels184
                bFontOffset = FontSndOffset184
                dwFontAddress = AddressOf(TableFont184)
            $endif
        Case 185
            $ifdef FontSnd185
                bFontSpace = FontSndSpace185
                bFontPixels = FontSndPixels185
                bFontOffset = FontSndOffset185
                dwFontAddress = AddressOf(TableFont185)
            $endif
        Case 186
            $ifdef FontSnd186
                bFontSpace = FontSndSpace186
                bFontPixels = FontSndPixels186
                bFontOffset = FontSndOffset186
                dwFontAddress = AddressOf(TableFont186)
            $endif
        Case 187
            $ifdef FontSnd187
                bFontSpace = FontSndSpace187
                bFontPixels = FontSndPixels187
                bFontOffset = FontSndOffset187
                dwFontAddress = AddressOf(TableFont187)
            $endif
        Case 188
            $ifdef FontSnd188
                bFontSpace = FontSndSpace188
                bFontPixels = FontSndPixels188
                bFontOffset = FontSndOffset188
                dwFontAddress = AddressOf(TableFont188)
            $endif
        Case 189
            $ifdef FontSnd189
                bFontSpace = FontSndSpace189
                bFontPixels = FontSndPixels189
                bFontOffset = FontSndOffset189
                dwFontAddress = AddressOf(TableFont189)
            $endif
        Case 190
            $ifdef FontSnd190
                bFontSpace = FontSndSpace190
                bFontPixels = FontSndPixels190
                bFontOffset = FontSndOffset190
                dwFontAddress = AddressOf(TableFont190)
            $endif
        Case 191
            $ifdef FontSnd191
                bFontSpace = FontSndSpace191
                bFontPixels = FontSndPixels191
                bFontOffset = FontSndOffset191
                dwFontAddress = AddressOf(TableFont191)
            $endif
        Case 192
            $ifdef FontSnd192
                bFontSpace = FontSndSpace192
                bFontPixels = FontSndPixels192
                bFontOffset = FontSndOffset192
                dwFontAddress = AddressOf(TableFont192)
            $endif
        Case 193
            $ifdef FontSnd193
                bFontSpace = FontSndSpace193
                bFontPixels = FontSndPixels193
                bFontOffset = FontSndOffset193
                dwFontAddress = AddressOf(TableFont193)
            $endif
        Case 194
            $ifdef FontSnd194
                bFontSpace = FontSndSpace194
                bFontPixels = FontSndPixels194
                bFontOffset = FontSndOffset194
                dwFontAddress = AddressOf(TableFont194)
            $endif
        Case 195
            $ifdef FontSnd195
                bFontSpace = FontSndSpace195
                bFontPixels = FontSndPixels195
                bFontOffset = FontSndOffset195
                dwFontAddress = AddressOf(TableFont195)
            $endif
        Case 196
            $ifdef FontSnd196
                bFontSpace = FontSndSpace196
                bFontPixels = FontSndPixels196
                bFontOffset = FontSndOffset196
                dwFontAddress = AddressOf(TableFont196)
            $endif
        Case 197
            $ifdef FontSnd197
                bFontSpace = FontSndSpace197
                bFontPixels = FontSndPixels197
                bFontOffset = FontSndOffset197
                dwFontAddress = AddressOf(TableFont197)
            $endif
        Case 198
            $ifdef FontSnd198
                bFontSpace = FontSndSpace198
                bFontPixels = FontSndPixels198
                bFontOffset = FontSndOffset198
                dwFontAddress = AddressOf(TableFont198)
            $endif
        Case 199
            $ifdef FontSnd199
                bFontSpace = FontSndSpace199
                bFontPixels = FontSndPixels199
                bFontOffset = FontSndOffset199
                dwFontAddress = AddressOf(TableFont199)
            $endif
        Case 200
            $ifdef FontSnd200
                bFontSpace = FontSndSpace200
                bFontPixels = FontSndPixels200
                bFontOffset = FontSndOffset200
                dwFontAddress = AddressOf(TableFont200)
            $endif
        Case 201
            $ifdef FontSnd201
                bFontSpace = FontSndSpace201
                bFontPixels = FontSndPixels201
                bFontOffset = FontSndOffset201
                dwFontAddress = AddressOf(TableFont201)
            $endif
        Case 202
            $ifdef FontSnd202
                bFontSpace = FontSndSpace202
                bFontPixels = FontSndPixels202
                bFontOffset = FontSndOffset202
                dwFontAddress = AddressOf(TableFont202)
            $endif
        Case 203
            $ifdef FontSnd203
                bFontSpace = FontSndSpace203
                bFontPixels = FontSndPixels203
                bFontOffset = FontSndOffset203
                dwFontAddress = AddressOf(TableFont203)
            $endif
        Case 204
            $ifdef FontSnd204
                bFontSpace = FontSndSpace204
                bFontPixels = FontSndPixels204
                bFontOffset = FontSndOffset204
                dwFontAddress = AddressOf(TableFont204)
            $endif
        Case 205
            $ifdef FontSnd205
                bFontSpace = FontSndSpace205
                bFontPixels = FontSndPixels205
                bFontOffset = FontSndOffset205
                dwFontAddress = AddressOf(TableFont205)
            $endif
        Case 206
            $ifdef FontSnd206
                bFontSpace = FontSndSpace206
                bFontPixels = FontSndPixels206
                bFontOffset = FontSndOffset206
                dwFontAddress = AddressOf(TableFont206)
            $endif
        Case 207
            $ifdef FontSnd207
                bFontSpace = FontSndSpace207
                bFontPixels = FontSndPixels207
                bFontOffset = FontSndOffset207
                dwFontAddress = AddressOf(TableFont207)
            $endif
        Case 208
            $ifdef FontSnd208
                bFontSpace = FontSndSpace208
                bFontPixels = FontSndPixels208
                bFontOffset = FontSndOffset208
                dwFontAddress = AddressOf(TableFont208)
            $endif
        Case 209
            $ifdef FontSnd209
                bFontSpace = FontSndSpace209
                bFontPixels = FontSndPixels209
                bFontOffset = FontSndOffset209
                dwFontAddress = AddressOf(TableFont209)
            $endif
        Case 210
            $ifdef FontSnd210
                bFontSpace = FontSndSpace210
                bFontPixels = FontSndPixels210
                bFontOffset = FontSndOffset210
                dwFontAddress = AddressOf(TableFont210)
            $endif
        Case 211
            $ifdef FontSnd211
                bFontSpace = FontSndSpace211
                bFontPixels = FontSndPixels211
                bFontOffset = FontSndOffset211
                dwFontAddress = AddressOf(TableFont211)
            $endif
        Case 212
            $ifdef FontSnd212
                bFontSpace = FontSndSpace212
                bFontPixels = FontSndPixels212
                bFontOffset = FontSndOffset212
                dwFontAddress = AddressOf(TableFont212)
            $endif
        Case 213
            $ifdef FontSnd213
                bFontSpace = FontSndSpace213
                bFontPixels = FontSndPixels213
                bFontOffset = FontSndOffset213
                dwFontAddress = AddressOf(TableFont213)
            $endif
        Case 214
            $ifdef FontSnd214
                bFontSpace = FontSndSpace214
                bFontPixels = FontSndPixels214
                bFontOffset = FontSndOffset214
                dwFontAddress = AddressOf(TableFont214)
            $endif
        Case 215
            $ifdef FontSnd215
                bFontSpace = FontSndSpace215
                bFontPixels = FontSndPixels215
                bFontOffset = FontSndOffset215
                dwFontAddress = AddressOf(TableFont215)
            $endif
        Case 216
            $ifdef FontSnd216
                bFontSpace = FontSndSpace216
                bFontPixels = FontSndPixels216
                bFontOffset = FontSndOffset216
                dwFontAddress = AddressOf(TableFont216)
            $endif
        Case 217
            $ifdef FontSnd217
                bFontSpace = FontSndSpace217
                bFontPixels = FontSndPixels217
                bFontOffset = FontSndOffset217
                dwFontAddress = AddressOf(TableFont217)
            $endif
        Case 218
            $ifdef FontSnd218
                bFontSpace = FontSndSpace218
                bFontPixels = FontSndPixels218
                bFontOffset = FontSndOffset218
                dwFontAddress = AddressOf(TableFont218)
            $endif
        Case 219
            $ifdef FontSnd219
                bFontSpace = FontSndSpace219
                bFontPixels = FontSndPixels219
                bFontOffset = FontSndOffset219
                dwFontAddress = AddressOf(TableFont219)
            $endif
        Case 220
            $ifdef FontSnd220
                bFontSpace = FontSndSpace220
                bFontPixels = FontSndPixels220
                bFontOffset = FontSndOffset220
                dwFontAddress = AddressOf(TableFont220)
            $endif
        Case 221
            $ifdef FontSnd221
                bFontSpace = FontSndSpace221
                bFontPixels = FontSndPixels221
                bFontOffset = FontSndOffset221
                dwFontAddress = AddressOf(TableFont221)
            $endif
        Case 222
            $ifdef FontSnd222
                bFontSpace = FontSndSpace222
                bFontPixels = FontSndPixels222
                bFontOffset = FontSndOffset222
                dwFontAddress = AddressOf(TableFont222)
            $endif
        Case 223
            $ifdef FontSnd223
                bFontSpace = FontSndSpace223
                bFontPixels = FontSndPixels223
                bFontOffset = FontSndOffset223
                dwFontAddress = AddressOf(TableFont223)
            $endif
        Case 224
            $ifdef FontSnd224
                bFontSpace = FontSndSpace224
                bFontPixels = FontSndPixels224
                bFontOffset = FontSndOffset224
                dwFontAddress = AddressOf(TableFont224)
            $endif
        Case 225
            $ifdef FontSnd225
                bFontSpace = FontSndSpace225
                bFontPixels = FontSndPixels225
                bFontOffset = FontSndOffset225
                dwFontAddress = AddressOf(TableFont225)
            $endif
        Case 226
            $ifdef FontSnd226
                bFontSpace = FontSndSpace226
                bFontPixels = FontSndPixels226
                bFontOffset = FontSndOffset226
                dwFontAddress = AddressOf(TableFont226)
            $endif
        Case 227
            $ifdef FontSnd227
                bFontSpace = FontSndSpace227
                bFontPixels = FontSndPixels227
                bFontOffset = FontSndOffset227
                dwFontAddress = AddressOf(TableFont227)
            $endif
        Case 228
            $ifdef FontSnd228
                bFontSpace = FontSndSpace228
                bFontPixels = FontSndPixels228
                bFontOffset = FontSndOffset228
                dwFontAddress = AddressOf(TableFont228)
            $endif
        Case 229
            $ifdef FontSnd229
                bFontSpace = FontSndSpace229
                bFontPixels = FontSndPixels229
                bFontOffset = FontSndOffset229
                dwFontAddress = AddressOf(TableFont229)
            $endif
        Case 230
            $ifdef FontSnd230
                bFontSpace = FontSndSpace230
                bFontPixels = FontSndPixels230
                bFontOffset = FontSndOffset230
                dwFontAddress = AddressOf(TableFont230)
            $endif
        Case 231
            $ifdef FontSnd231
                bFontSpace = FontSndSpace231
                bFontPixels = FontSndPixels231
                bFontOffset = FontSndOffset231
                dwFontAddress = AddressOf(TableFont231)
            $endif
        Case 232
            $ifdef FontSnd232
                bFontSpace = FontSndSpace232
                bFontPixels = FontSndPixels232
                bFontOffset = FontSndOffset232
                dwFontAddress = AddressOf(TableFont232)
            $endif
        Case 233
            $ifdef FontSnd233
                bFontSpace = FontSndSpace233
                bFontPixels = FontSndPixels233
                bFontOffset = FontSndOffset233
                dwFontAddress = AddressOf(TableFont233)
            $endif
        Case 234
            $ifdef FontSnd234
                bFontSpace = FontSndSpace234
                bFontPixels = FontSndPixels234
                bFontOffset = FontSndOffset234
                dwFontAddress = AddressOf(TableFont234)
            $endif
        Case 235
            $ifdef FontSnd235
                bFontSpace = FontSndSpace235
                bFontPixels = FontSndPixels235
                bFontOffset = FontSndOffset235
                dwFontAddress = AddressOf(TableFont235)
            $endif
        Case 236
            $ifdef FontSnd236
                bFontSpace = FontSndSpace236
                bFontPixels = FontSndPixels236
                bFontOffset = FontSndOffset236
                dwFontAddress = AddressOf(TableFont236)
            $endif
        Case 237
            $ifdef FontSnd237
                bFontSpace = FontSndSpace237
                bFontPixels = FontSndPixels237
                bFontOffset = FontSndOffset237
                dwFontAddress = AddressOf(TableFont237)
            $endif
        Case 238
            $ifdef FontSnd238
                bFontSpace = FontSndSpace238
                bFontPixels = FontSndPixels238
                bFontOffset = FontSndOffset238
                dwFontAddress = AddressOf(TableFont238)
            $endif
        Case 239
            $ifdef FontSnd239
                bFontSpace = FontSndSpace239
                bFontPixels = FontSndPixels239
                bFontOffset = FontSndOffset239
                dwFontAddress = AddressOf(TableFont239)
            $endif
        Case 240
            $ifdef FontSnd240
                bFontSpace = FontSndSpace240
                bFontPixels = FontSndPixels240
                bFontOffset = FontSndOffset240
                dwFontAddress = AddressOf(TableFont240)
            $endif
        Case 241
            $ifdef FontSnd241
                bFontSpace = FontSndSpace241
                bFontPixels = FontSndPixels241
                bFontOffset = FontSndOffset241
                dwFontAddress = AddressOf(TableFont241)
            $endif
        Case 242
            $ifdef FontSnd242
                bFontSpace = FontSndSpace242
                bFontPixels = FontSndPixels242
                bFontOffset = FontSndOffset242
                dwFontAddress = AddressOf(TableFont242)
            $endif
        Case 243
            $ifdef FontSnd243
                bFontSpace = FontSndSpace243
                bFontPixels = FontSndPixels243
                bFontOffset = FontSndOffset243
                dwFontAddress = AddressOf(TableFont243)
            $endif
        Case 244
            $ifdef FontSnd244
                bFontSpace = FontSndSpace244
                bFontPixels = FontSndPixels244
                bFontOffset = FontSndOffset244
                dwFontAddress = AddressOf(TableFont244)
            $endif
        Case 245
            $ifdef FontSnd245
                bFontSpace = FontSndSpace245
                bFontPixels = FontSndPixels245
                bFontOffset = FontSndOffset245
                dwFontAddress = AddressOf(TableFont245)
            $endif
        Case 246
            $ifdef FontSnd246
                bFontSpace = FontSndSpace246
                bFontPixels = FontSndPixels246
                bFontOffset = FontSndOffset246
                dwFontAddress = AddressOf(TableFont246)
            $endif
        Case 247
            $ifdef FontSnd247
                bFontSpace = FontSndSpace247
                bFontPixels = FontSndPixels247
                bFontOffset = FontSndOffset247
                dwFontAddress = AddressOf(TableFont247)
            $endif
        Case 248
            $ifdef FontSnd248
                bFontSpace = FontSndSpace248
                bFontPixels = FontSndPixels248
                bFontOffset = FontSndOffset248
                dwFontAddress = AddressOf(TableFont248)
            $endif
        Case 249
            $ifdef FontSnd249
                bFontSpace = FontSndSpace249
                bFontPixels = FontSndPixels249
                bFontOffset = FontSndOffset249
                dwFontAddress = AddressOf(TableFont249)
            $endif
        Case 250
            $ifdef FontSnd250
                bFontSpace = FontSndSpace250
                bFontPixels = FontSndPixels250
                bFontOffset = FontSndOffset250
                dwFontAddress = AddressOf(TableFont250)
            $endif
        Case 251
            $ifdef FontSnd251
                bFontSpace = FontSndSpace251
                bFontPixels = FontSndPixels251
                bFontOffset = FontSndOffset251
                dwFontAddress = AddressOf(TableFont251)
            $endif
        Case 252
            $ifdef FontSnd252
                bFontSpace = FontSndSpace252
                bFontPixels = FontSndPixels252
                bFontOffset = FontSndOffset252
                dwFontAddress = AddressOf(TableFont252)
            $endif
        Case 253
            $ifdef FontSnd253
                bFontSpace = FontSndSpace253
                bFontPixels = FontSndPixels253
                bFontOffset = FontSndOffset253
                dwFontAddress = AddressOf(TableFont253)
            $endif
        Case 254
            $ifdef FontSnd254
                bFontSpace = FontSndSpace254
                bFontPixels = FontSndPixels254
                bFontOffset = FontSndOffset254
                dwFontAddress = AddressOf(TableFont254)
            $endif
        Case 255
            $ifdef FontSnd255
                bFontSpace = FontSndSpace255
                bFontPixels = FontSndPixels255
                bFontOffset = FontSndOffset255
                dwFontAddress = AddressOf(TableFont255)
            $endif 
    EndSelect    
EndProc
