Global Dim bInitiated As Byte Shared

$if MEM_COM = SPI_1 Or MEM_COM = SPI_2 Or MEM_COM = SPI_SW
    Output MEM_CS : Output MEM_MOSI       
    Output MEM_SCLK : Input MEM_MISO
    PinSet MEM_CS
$endif  

$define TftBmpMemFormat() TftBmpFlashFormat()
$define TftBmpMemSet() TftBmpFlashSet()


'Get BMP file name from FLASH memory index
'wFileIndex = File index (zero-based) in FLASH memory
'Return parameter: - File Name as string
'                  - 0 index not found
'Example: TftBmpMemFileName(4) --> Return 5th file's name

Proc TftBmpMemFileName(wFileIndex As Word), String * 12
    dwTftTemp1_Bmp = (MEM_LAST_DATA_PAGE + ((8192 * (MEM_FILE_COUNT / 256)) / 2) / MEM_PAGE_SIZE) * MEM_PAGE_SIZE 
    dwTftTemp1_Bmp = dwTftTemp1_Bmp + (wFileIndex * 16)
    Clear Result
   
    For bTftCounter1_Bmp = 0 To 12
        Result[bTftCounter1_Bmp] = TftBmpFlashByteRead(dwTftTemp1_Bmp + bTftCounter1_Bmp) 
        Select Result[bTftCounter1_Bmp] 
            Case 32 To 126
                Continue
            Case 0
                ExitProc
            Case Else
                Clear Result
                ExitProc  
        EndSelect
    Next
EndProc

'Print a BMP file from FLASH memory 
'sFileName = File name of the image to print, should be less than 13 characters, including dot and file extension
'wX_Pos = x position for the lower left corner of the image
'wY_Pos = y position for the lower left corner of the image
'Return parameter: 0 = No error
'                  1 = File not found 
'Example: TftBmpFromMemName("PIC001.BMP",20,30) --> Print file with name "PIC001.BMP" at position 20,30

'Note: there's no image processing so the BMP file should be:
'      - smaller than the screen width and height
'      - the color depth should be 16 bit RGB (565)
'      - the row direction is bottom to top (default)
'      - no compression
'You can use different tools to conver the file, one such tool is: 
'https://online-converting.com/image/convert2bmp/#
'Select the correct format, then drop the files, wait and download the converted file
'Or you can use the included Library converter tool

Proc TftBmpFromMemName(sFileName As String * 12, wX_Pos As Word, wY_Pos As Word), Byte
    dwTftTemp1_Bmp = (MEM_LAST_DATA_PAGE + ((8192 * (MEM_FILE_COUNT / 256)) / 2) / MEM_PAGE_SIZE) * MEM_PAGE_SIZE 
    sFileName = ToUpper(sFileName) 
    
    For wTftCounter1_Bmp = 0 To MEM_FILE_COUNT - 1
        PinClear MEM_SCLK : PinClear MEM_CS
            TftSendMem(0x03)
            TftSendMem(dwTftTemp1_Bmp.Byte2)
            TftSendMem(dwTftTemp1_Bmp.Byte1)
            TftSendMem(dwTftTemp1_Bmp.Byte0)
            For bTftCounter2_Bmp = 0 To 11
                sTftTemp[bTftCounter2_Bmp] = TftRecMem()
            Next
        PinSet MEM_CS
        If sTftTemp = sFileName Then
            TftBmpFromMemIndex(wTftCounter1_Bmp, wX_Pos, wY_Pos)
            Result = 0
            ExitProc 
        EndIf
        dwTftTemp1_Bmp = dwTftTemp1_Bmp + 16
    Next

    Result = 1
EndProc

'Print a BMP file from FLASH memory 
'wFileIndex = File index (zero-based) in FLASH memory
'wX_Pos = x position for the lower left corner of the image
'wY_Pos = y position for the lower left corner of the image

'Example: TftBmpFromMemIndex(2,20,30) --> Print file number 2 at position 20,30

'Note: there's no image processing so the BMP file should be:
'      - smaller than the screen width and height
'      - the color depth should be 16 bit RGB (565)
'      - the row direction is bottom to top (default)
'      - no compression
'You can use different tools to conver the file, one such tool is: 
'https://online-converting.com/image/convert2bmp/#
'Select the correct format, then drop the files, wait and download the converted file 
'Or you can use the included Library converter tool

Proc TftBmpFromMemIndex(wFileIndex As Word, wX_Pos As Word, wY_Pos As Word)
    dwTftTemp1_Bmp = MEM_LAST_DATA_PAGE * MEM_PAGE_SIZE 
    dwTftTemp1_Bmp = dwTftTemp1_Bmp + (wFileIndex * 16) 
    dwTftAddressStart_Bmp = TftBmpFlashDwordRead(dwTftTemp1_Bmp) 
    dwTftAddressStop_Bmp = TftBmpFlashDwordRead(dwTftTemp1_Bmp + 4) 

    dwTftOffset_Bmp = TftBmpFlashDwordRead(dwTftAddressStart_Bmp + 10)  
    If TftBmpFlashByteRead(dwTftAddressStart_Bmp + 1) = "M" Then
        wTftWidth_Bmp = TftBmpFlashWordRead(dwTftAddressStart_Bmp + 18)   
        wTftHeight_Bmp = TftBmpFlashWordRead(dwTftAddressStart_Bmp + 22)    
    Else
        wTftWidth_Bmp = TftBmpFlashWordRead(dwTftAddressStart_Bmp + 18)   
        wTftHeight_Bmp = TftBmpFlashWordRead(dwTftAddressStart_Bmp + 20)  
    EndIf 
    dwTftSize_Bmp = TftBmpFlashDwordRead(dwTftAddressStart_Bmp + 34)
    
    bTftTrim_Bmp = (wTftWidth_Bmp + wTftWidth_Bmp) // 4
    dwTftOffset_Bmp = dwTftOffset_Bmp + dwTftAddressStart_Bmp 
        
    $if TFT_COM = SPI_1 Or TFT_COM = SPI_2 Or TFT_COM = SPI_SW
        $if _TFT_MOSI = _MEM_MOSI Or _TFT_MISO = _MEM_MISO Or _TFT_SCLK = _MEM_SCLK
            For wTftCounter1_Bmp = 0 To wTftHeight_Bmp - 1
                wTftCounter2_Bmp = 0
                Do 
                    wTftColor_Bmp.Byte0 = TftBmpFlashDwordRead(dwTftOffset_Bmp)
                    Inc dwTftOffset_Bmp
                    wTftColor_Bmp.Byte1 = TftBmpFlashDwordRead(dwTftOffset_Bmp)
                    Inc dwTftOffset_Bmp
                    TftPixel(wTftCounter2_Bmp + wX_Pos, wTftCounter1_Bmp + wY_Pos, wTftColor_Bmp)
                    Inc wTftCounter2_Bmp   
                Loop While wTftCounter2_Bmp < wTftWidth_Bmp 
            
                dwTftOffset_Bmp = dwTftOffset_Bmp + bTftTrim_Bmp
            Next 
        $else
            Dim wTemp As Word
            Dim bTemp As Byte
            Dim bTemp0 As wTemp.Byte0
            Dim bTemp1 As wTemp.Byte1
            
            $if TftPixelCheck = 1
                If wX_Pos < TftScreenWidth And wY_Pos < TftScreenHeight Then
                    wTempHorizontal = wX_Pos + wTftWidth_Bmp - 1 : wTempVertical = wY_Pos + wTftHeight_Bmp - 1
                    If wTempHorizontal >= TftScreenWidth Then wTempHorizontal = TftScreenWidth - 1
                    If wTempVertical >= TftScreenHeight Then wTempVertical = TftScreenHeight - 1  
                    TftSetWindow(wX_Pos, wTempHorizontal, wY_Pos, wTempVertical)  
                Else
                    ExitProc              
                EndIf   
            $else
                TftSetWindow(wX_Pos, wX_Pos + wTftWidth_Bmp - 1, wY_Pos, wY_Pos + wTftHeight_Bmp - 1)
            $endif
            TftInitiateDataRam()
        
            PinClear MEM_SCLK : PinClear MEM_CS
            TftSendMem(0x03)
            TftSendMem(dwTftOffset_Bmp.Byte2)
            TftSendMem(dwTftOffset_Bmp.Byte1)
            TftSendMem(dwTftOffset_Bmp.Byte0)
            
            bInitiated = 0
            For wTftCounter1_Bmp = 0 To wTftHeight_Bmp - 1
                wTftCounter2_Bmp = 0
                Do 
                    bTemp0 = TftRecMem()
                    bTemp1 = TftRecMem()
                    
                    $if TFT_COM = SPI_1
                        $if TFT_IC = ILI9488 
                            Rgb16ToRgb18(wTemp) 
                            TftSend(dColor.Byte2)
                            TftSend(dColor.Byte1)
                            TftSend(dColor.Byte0)
                        $else
                            $if McuType = 0    
                                If bInitiated = 0 Then
                                    bInitiated = 1
                                Else
                                    While SSP1STAT.0 = 0 : Wend 
                                EndIf
                                bTemp = SSP1BUF
                                SSP1BUF = bTemp1
                                While SSP1STAT.0 = 0 : Wend  
                                bTemp = SSP1BUF
                                SSP1BUF = bTemp0
                            $else
                                If bInitiated = 0 Then
                                    bInitiated = 1
                                Else
                                    While SPI1STAT.0 = 0 : Wend  
                                EndIf
                                bTemp = SPI1BUF
                                SPI1BUF = bTemp1
                                While SPI1STAT.0 = 0 : Wend
                                bTemp = SPI1BUF
                                SPI1BUF = bTemp0
                            $endif   
                        $endif 
                    $elseif TFT_COM = SPI_2
                        $if TFT_IC = ILI9488 
                            Rgb16ToRgb18(wTemp) 
                            TftSend(dColor.Byte2)
                            TftSend(dColor.Byte1)
                            TftSend(dColor.Byte0)
                        $else        
                            $if McuType = 0
                                If bInitiated = 0 Then
                                    bInitiated = 1
                                Else
                                    While SSP2STAT.0 = 0 : Wend 
                                EndIf
                                bTemp = SSP2BUF
                                SSP2BUF = bTemp1
                                While SSP2STAT.0 = 0 : Wend
                                bTemp = SSP2BUF
                                SSP2BUF = bTemp0
                            $else
                                If bInitiated = 0 Then
                                    bInitiated = 1
                                Else
                                    While SPI2STAT.0 = 0 : Wend 
                                EndIf
                                bTemp = SPI2BUF
                                SPI2BUF = bTemp1
                                While SPI2STAT.0 = 0 : Wend
                                bTemp = SPI2BUF
                                SPI2BUF = bTemp0
                            $endif  
                        $endif 
                    $elseif TFT_COM = SPI_SW 
                        $if TFT_IC = ILI9488 
                            Rgb16ToRgb18(wTemp) 
                            TftSend(dColor.Byte2)
                            TftSend(dColor.Byte1)
                            TftSend(dColor.Byte0)
                        $else
                            TftSend(bTemp1)
                            TftSend(bTemp0)
                        $endif     
                    $endif
                    
                    Inc wTftCounter2_Bmp   
                Loop While wTftCounter2_Bmp < wTftWidth_Bmp 
        
                Select bTftTrim_Bmp
                    Case 1
                        TftRecMem()
                    Case 2
                        TftRecMem()
                        TftRecMem()
                    Case 3
                        TftRecMem()
                        TftRecMem()
                        TftRecMem()
                EndSelect  
            Next  
            PinSet MEM_CS 
            
            $if TFT_IC = ILI9488 
                'Do nothing
            $else 
                $if TFT_COM = SPI_1
                    $if McuType = 0
                        While SSP1STAT.0 = 0 : Wend  
                        bTemp = SSP1BUF
                    $else
                        While SPI1STAT.0 = 0 : Wend
                        bTemp = SPI1BUF
                    $endif
                $elseif TFT_COM = SPI_2 
                    $if McuType = 0
                        While SSP2STAT.0 = 0 : Wend
                        bTemp = SSP2BUF
                    $else
                        While SPI2STAT.0 = 0 : Wend
                        bTemp = SPI2BUF
                    $endif
                $endif
            $endif
            
            TftStopDataRam() 
        $endif       
    $elseif TFT_COM = PAR_2x8b Or TFT_COM = PAR_16b
        $if TftPixelCheck = 1
            If wX_Pos < TftScreenWidth And wY_Pos < TftScreenHeight Then
                wTempHorizontal = wX_Pos + wTftWidth_Bmp - 1 : wTempVertical = wY_Pos + wTftHeight_Bmp - 1
                If wTempHorizontal >= TftScreenWidth Then wTempHorizontal = TftScreenWidth - 1
                If wTempVertical >= TftScreenHeight Then wTempVertical = TftScreenHeight - 1  
                TftSetWindow(wX_Pos, wTempHorizontal, wY_Pos, wTempVertical)  
            Else
                ExitProc              
            EndIf   
        $else
            TftSetWindow(wX_Pos, wX_Pos + wTftWidth_Bmp - 1, wY_Pos, wY_Pos + wTftHeight_Bmp - 1)
        $endif
        TftInitiateDataRam()
        
        PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x03)
        TftSendMem(dwTftOffset_Bmp.Byte2)
        TftSendMem(dwTftOffset_Bmp.Byte1)
        TftSendMem(dwTftOffset_Bmp.Byte0)
     
        For wTftCounter1_Bmp = 0 To wTftHeight_Bmp - 1
            wTftCounter2_Bmp = 0
            Do 
                $if McuType = 0
                    TftDataLowByte = TftRecMem()
                    TftDataHighByte = TftRecMem()
                $else
                    TftDataWord.Byte0 = TftRecMem() 
                    TftDataWord.Byte1 = TftRecMem()
                $endif
                PinClear TFT_WR : PinSet TFT_WR 
                Inc wTftCounter2_Bmp   
            Loop While wTftCounter2_Bmp < wTftWidth_Bmp 
        
            Select bTftTrim_Bmp
                Case 1
                    TftRecMem()
                Case 2
                    TftRecMem()
                    TftRecMem()
                Case 3
                    TftRecMem()
                    TftRecMem()
                    TftRecMem()
            EndSelect  
        Next  
        PinSet MEM_CS 
        TftStopDataRam() 
    $endif
EndProc 

'This procedure initializes FLASH memory
Proc TftBmpFlashSet()
    Global Dim bMemPageArray[MEM_PAGE_SIZE] As Byte Shared
    
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x66)
    PinSet MEM_CS
    DelayUS 100
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x99)
    PinSet MEM_CS
    DelayMS 30
    TftBmpFlashWriteUnlock()
EndProc

'Flash write unlock
Proc TftBmpFlashWriteUnlock()
    TftBmpFlashWriteEnable()
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x98)
    PinSet MEM_CS
EndProc

'Flash write enable
Proc TftBmpFlashWriteEnable()
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x06)
    PinSet MEM_CS
EndProc

'Format Flash memory
Proc TftBmpFlashFormat()
    'Chip erase mode
    TftBmpFlashWriteEnable()
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0xC7)
    PinSet MEM_CS
    DelayMS 1000
EndProc

'Flash memory sector erase
Proc TftBmpFlashSectorErase(wSector As Word)
    TftBmpFlashWriteEnable()
    dwTftCounter1_Bmp =  wSector * MEM_SECTOR_SIZE
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x20)
        TftSendMem(dwTftCounter1_Bmp.Byte2)
        TftSendMem(dwTftCounter1_Bmp.Byte1)
        TftSendMem(dwTftCounter1_Bmp.Byte0)
    PinSet MEM_CS
EndProc

'Write page in FLASH
'dwAddress = page number
Proc TftBmpFlashPageWrite(dwAddress As Dword)
    Global Dim bMemPageArray[MEM_PAGE_SIZE] As Byte Shared
    
    Dim wCounter1 As Word
    dwAddress = dwAddress * MEM_PAGE_SIZE
    TftBmpFlashWriteEnable()

    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x02)
        TftSendMem(dwAddress.Byte2)
        TftSendMem(dwAddress.Byte1)
        TftSendMem(dwAddress.Byte0)
        For wCounter1 = 0 To MEM_PAGE_SIZE - 1
            TftSendMem(bMemPageArray[wCounter1])
        Next
    PinSet MEM_CS
    DelayMS MEM_WRITE_DELAY
EndProc

'Read byte from flash address "dwAddress"
Proc TftBmpFlashByteRead(dwAddress As Dword), Byte
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x03)
        TftSendMem(dwAddress.Byte2)
        TftSendMem(dwAddress.Byte1)
        TftSendMem(dwAddress.Byte0)
        Result = TftRecMem()
    PinSet MEM_CS
EndProc

'Read word from flash address "dwAddress"
Proc TftBmpFlashWordRead(dwAddress As Dword), Word
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x03)
        TftSendMem(dwAddress.Byte2)
        TftSendMem(dwAddress.Byte1)
        TftSendMem(dwAddress.Byte0)
        Result.Byte0 = TftRecMem()
        Result.Byte1 = TftRecMem()
    PinSet MEM_CS
EndProc

'Read dword from flash address "dwAddress"
Proc TftBmpFlashDwordRead(dwAddress As Dword), Dword
    PinClear MEM_SCLK : PinClear MEM_CS
        TftSendMem(0x03)
        TftSendMem(dwAddress.Byte2)
        TftSendMem(dwAddress.Byte1)
        TftSendMem(dwAddress.Byte0)
        Result.Byte0 = TftRecMem()
        Result.Byte1 = TftRecMem()
        Result.Byte2 = TftRecMem()
        Result.Byte3 = TftRecMem()
    PinSet MEM_CS
EndProc

Proc TftSendMem(bData As Byte)
    $if MEM_COM = SPI_1
        $if McuType = 0
            SSP1BUF = bData
            While SSP1STAT.0 = 0 : Wend
            bData = SSP1BUF
        $else
            SPI1BUF = bData
            While SPI1STAT.0 = 0 : Wend
            bData = SPI1BUF
        $endif
    $elseif MEM_COM = SPI_2
        $if McuType = 0
            SSP2BUF = bData
            While SSP2STAT.0 = 0 : Wend
            bData = SSP2BUF
        $else
            SPI2BUF = bData
            While SPI2STAT.0 = 0 : Wend
            bData = SPI2BUF
        $endif
    $elseif MEM_COM = SPI_SW
        MEM_MOSI = bData.7 : PinSet MEM_SCLK : PinClear MEM_SCLK 
        MEM_MOSI = bData.6 : PinSet MEM_SCLK : PinClear MEM_SCLK 
        MEM_MOSI = bData.5 : PinSet MEM_SCLK : PinClear MEM_SCLK 
        MEM_MOSI = bData.4 : PinSet MEM_SCLK : PinClear MEM_SCLK 
        MEM_MOSI = bData.3 : PinSet MEM_SCLK : PinClear MEM_SCLK 
        MEM_MOSI = bData.2 : PinSet MEM_SCLK : PinClear MEM_SCLK 
        MEM_MOSI = bData.1 : PinSet MEM_SCLK : PinClear MEM_SCLK 
        MEM_MOSI = bData.0 : PinSet MEM_SCLK : PinClear MEM_SCLK 
    $endif
EndProc

Proc TftRecMem(), Byte
    $if MEM_COM = SPI_1 
        $if McuType = 0
            SSP1BUF = 0x00
            While SSP1STAT.0 = 0 : Wend
            Result = SSP1BUF
        $else
            SPI1BUF = 0x00
            While SPI1STAT.0 = 0 : Wend
            Result = SPI1BUF
        $endif
    $elseif MEM_COM = SPI_2
        $if McuType = 0
            SSP2BUF = 0x00
            While SSP2STAT.0 = 0 : Wend
            Result = SSP2BUF
        $else
            SPI2BUF = 0x00
            While SPI2STAT.0 = 0 : Wend
            Result = SPI2BUF
        $endif
    $elseif MEM_COM = SPI_SW
        PinClear MEM_MOSI 
        PinSet MEM_SCLK : Result.7 = MEM_MISO : PinClear MEM_SCLK 
        PinSet MEM_SCLK : Result.6 = MEM_MISO : PinClear MEM_SCLK 
        PinSet MEM_SCLK : Result.5 = MEM_MISO : PinClear MEM_SCLK 
        PinSet MEM_SCLK : Result.4 = MEM_MISO : PinClear MEM_SCLK
        PinSet MEM_SCLK : Result.3 = MEM_MISO : PinClear MEM_SCLK
        PinSet MEM_SCLK : Result.2 = MEM_MISO : PinClear MEM_SCLK
        PinSet MEM_SCLK : Result.1 = MEM_MISO : PinClear MEM_SCLK
        PinSet MEM_SCLK : Result.0 = MEM_MISO : PinClear MEM_SCLK
    $endif
EndProc
