'   /\\\\\\\\\
'  /\\\///////\\\
'  \/\\\     \/\\\                                                 /\\\          /\\\
'   \/\\\\\\\\\\\/        /\\\\\     /\\\\\\\\\\     /\\\\\\\\   /\\\\\\\\\\\  /\\\\\\\\\\\  /\\\\\\\\\
'    \/\\\//////\\\      /\\\///\\\  \/\\\//////    /\\\/////\\\ \////\\\////  \////\\\////  \////////\\\
'     \/\\\    \//\\\    /\\\  \//\\\ \/\\\\\\\\\\  /\\\\\\\\\\\     \/\\\         \/\\\        /\\\\\\\\\\
'      \/\\\     \//\\\  \//\\\  /\\\  \////////\\\ \//\\///////      \/\\\ /\\     \/\\\ /\\   /\\\/////\\\
'       \/\\\      \//\\\  \///\\\\\/    /\\\\\\\\\\  \//\\\\\\\\\\    \//\\\\\      \//\\\\\   \//\\\\\\\\/\\
'        \///        \///     \/////     \//////////    \//////////      \/////        \/////     \////////\//
'                                  Let's find out together what makes a PIC Tick!
'
' FAT16/32 File system procedures
' Written by Les Johnson for the Positron16 BASIC compiler
'
' SD commands
'
    Symbol SD_cCMD0  = $40 + 0                              ' GO_IDLE_STATE
    Symbol SD_cCMD1  = $40 + 1                              ' SEND_OP_COND
    Symbol SD_cCMD8  = $40 + 8                              ' SEND_IF_COND
    Symbol SD_cCMD9  = $40 + 9                              ' SEND_CSD
    Symbol SD_cCMD10 = $40 + 10                             ' SEND_CID
    Symbol SD_cCMD12 = $40 + 12                             ' STOP_TRANSMISSION
    Symbol SD_cCMD16 = $40 + 16                             ' SET_BLOCKLEN
    Symbol SD_cCMD17 = $40 + 17                             ' READ_SINGLE_BLOCK
    Symbol SD_cCMD18 = $40 + 18                             ' READ_MULTIPLE_BLOCK
    Symbol SD_cCMD23 = $40 + 23                             ' SET_BLOCK_COUNT
    Symbol SD_cCMD24 = $40 + 24                             ' WRITE_BLOCK
    Symbol SD_cCMD25 = $40 + 25                             ' WRITE_MULTIPLE_BLOCK
    Symbol SD_cCMD41 = $40 + 41                             ' SEND_OP_COND (ASD_cCMD)
    Symbol SD_cCMD55 = $40 + 55                             ' APP_SD_cCMD
    Symbol SD_cCMD58 = $40 + 58                             ' READ_OCR

$define SD_cMissing 1
'
' Set a timeout limit based on the oscillator
'
    Symbol SD_cTimeOutLimit = _xtal * 200
'
' Dir Search & Rename types
'
    $define FAT_cFile 0
    $define FAT_cDirectory 1
'
' Dir search directions
'
    $define FAT_cDirFirst 0
    $define FAT_cDirNext 1
    $define FAT_cDirPrevious 2
'
' ChDir commands
'
    $define FAT_cCdUp 0                         ' Change to parent directory
    $define FAT_cCdRoot 1                       ' Change to Root Directory
'
' Error codes
'
    $define cErrOK            $00
    $define cErrDiskFull      $01
    $define cErrNotFound      $02
    $define cErrRootDirFull   $03
    $define cErrDirNotEmpty   $04
    $define cErrExists        $05
    $define cErrInUse         $06
    $define cErrRWError       $07
    $define cErrFileNotOpen   $08
    $define cErrBeyondEOF     $09
    $define cErrNoResponse    $0A
    $define cErrInvalidFormat $0B
    $define cErrInvalidPath   $0C
'
' Card types
'
    $define SD_cMMC 1                           ' Multi Media Card (MMC)
    $define SD_cSD 2                            ' Secure Digital Card (SD)
    $define SD_cSDHC 4                          ' High Capacity Secure Digital Card (SDHC)
'
' FAT types
'
    $define FAT_cFAT16 1
    $define FAT_cFAT32 2

    $ifdef SD_Detect                                            ' Are we using the SD detect switch?
        Dim SD_tInserted As Bit                                 ' False if the SD card is removed or missing
    $endif    

    $ifndef True
        $define True 1
    $endif
    $ifndef False
        $define False 0
    $endif
    
    Dim Disk_tSlowSPI As Bit  
    Dim Disk_bCardType As Byte
        
    $if McuType = 0
        Dim WREG0 As Byte
        Dim WREG2 As Word
        Dim WREG3 As Word
    $endif
    
'--------------------------------------------------------------------------------------------------------------------------
' Receive a byte from the card using a software SPI interface.
' Input     : None
' Return    : Returns the byte read from the SPI bus
' Notes     : None
'
$if McuType = 1
Proc SD_ReceiveByte(), WREG0.Byte0              
$else
Proc SD_ReceiveByte(), WREG0              
$endif
    $if SD_COM = SPI_1 Or SD_COM = SPI_2
        If Disk_tSlowSPI = True Then
            PinSet SD_SDO
            PinSet SD_CLK : DelayUS 4 : WREG0.7 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.6 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.5 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.4 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.3 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.2 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.1 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.0 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
        Else
            $if SD_COM = SPI_1
                $if McuType = 1
                    SPI1BUF = 0xFF
                    While SPI1STAT.0 = 0 : Wend
                    WREG0.Byte0 = SPI1BUF  
                $else
                    SSP1BUF = 0xFF
                    While SSP1STAT.0 = 0 : Wend
                    WREG0 = SSP1BUF
                $endif
            $elseif SD_COM = SPI_2
                $if McuType = 1
                    SPI2BUF = 0xFF
                    While SPI2STAT.0 = 0 : Wend
                    WREG0.Byte0 = SPI2BUF  
                $else
                    SSP2BUF = 0xFF
                    While SSP2STAT.0 = 0 : Wend
                    WREG0 = SSP2BUF
                $endif
            $endif
        EndIf
    $elseif SD_COM = SPI_SW
        If Disk_tSlowSPI = True Then
            PinSet SD_SDO
            PinSet SD_CLK : DelayUS 4 : WREG0.7 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.6 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.5 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.4 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.3 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.2 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.1 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            PinSet SD_CLK : DelayUS 4 : WREG0.0 = SD_SDI : DelayUS 4 : PinClear SD_CLK : DelayUS 4
        Else
            PinSet SD_SDO
            PinSet SD_CLK : WREG0.7 = SD_SDI : PinClear SD_CLK 
            PinSet SD_CLK : WREG0.6 = SD_SDI : PinClear SD_CLK
            PinSet SD_CLK : WREG0.5 = SD_SDI : PinClear SD_CLK
            PinSet SD_CLK : WREG0.4 = SD_SDI : PinClear SD_CLK
            PinSet SD_CLK : WREG0.3 = SD_SDI : PinClear SD_CLK
            PinSet SD_CLK : WREG0.2 = SD_SDI : PinClear SD_CLK
            PinSet SD_CLK : WREG0.1 = SD_SDI : PinClear SD_CLK
            PinSet SD_CLK : WREG0.0 = SD_SDI : PinClear SD_CLK
        EndIf
    $endif
EndProc

Proc SD_RecByteBmp(), Byte
    If wTftCounter3_Bmp = 512 Then
        SD_ReceiveByte()
        SD_ReceiveByte()
        
        wSdExitCounter = SD_cTimeOutLimit
        $if McuType = 1
            Repeat 
                bTftTemp1_Bmp = SD_ReceiveByte()
                Dec wSdExitCounter
            Until bTftTemp1_Bmp = $FE Or wSdExitCounter = 0
        $else
            Repeat 
                Dec wSdExitCounter
            Until SD_ReceiveByte() = $FE Or wSdExitCounter = 0
        $endif 
        wTftCounter3_Bmp = 0
    EndIf
                      
    Result = SD_ReceiveByte() 
    Inc wTftCounter3_Bmp
EndProc

Proc TftBmpNewSector()
    If wTftCounter3_Bmp = 512 Then
        Inc dwTftCounter1_Bmp
        SD_ReadSector(dwTftCounter1_Bmp)
        wTftCounter3_Bmp = 0
    EndIf
EndProc


'--------------------------------------------------------------------------------------------------------------------------
' Wait for a specified response from card
' Input     : pRequired holds the response wo wait for
' Return    : Returns the response from the SD card
' Notes     :
'
Proc SD_SeekResponse(pRequired As WREG3.Byte0), Byte
    Dim wTimeout As WREG2 = $FFFF

    wTimeout = $FFFF
    Repeat
        Result = SD_ReceiveByte()
        Dec wTimeout
        If wTimeout = 0 Then Break
    Until Result = pRequired
EndProc

'--------------------------------------------------------------------------------------------------------------------------
' Send a byte to the card using a software SPI interface.
' Input     : pByte holds teh byte to write to the SPI bus
' Return    : None
' Notes     : None
'

$if McuType = 1
Proc SD_SendByte(pByte As WREG0.Byte0)           
$else
Proc SD_SendByte(pByte As WREG0)            
$endif
    Dim bDummy As Byte
    
    $if SD_COM = SPI_1 Or SD_COM = SPI_2
        If Disk_tSlowSPI = True Then
            SD_SDO = WREG0.7 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.6 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.5 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.4 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.3 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.2 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.1 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.0 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
        Else
            $if SD_COM = SPI_1
                $if McuType = 1
                    SPI1BUF = pByte
                    While SPI1STAT.0 = 0 : Wend
                    bDummy = SPI1BUF 
                $else
                    SSP1BUF = pByte
                    While SSP1STAT.0 = 0 : Wend
                    bDummy = SSP1BUF
                $endif 
            $elseif SD_COM = SPI_2  
                $if McuType = 1
                    SPI2BUF = pByte
                    While SPI2STAT.0 = 0 : Wend
                    bDummy = SPI2BUF 
                $else
                    SSP2BUF = pByte
                    While SSP2STAT.0 = 0 : Wend
                    bDummy = SSP2BUF
                $endif 
            $endif
        EndIf
    $elseif SD_COM = SPI_SW
        If Disk_tSlowSPI = True Then
            SD_SDO = WREG0.7 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.6 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.5 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.4 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.3 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.2 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.1 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4
            SD_SDO = WREG0.0 : DelayUS 4 : PinSet SD_CLK : DelayUS 4 : PinClear SD_CLK : DelayUS 4 
        Else
            SD_SDO = WREG0.7 : PinSet SD_CLK : PinClear SD_CLK 
            SD_SDO = WREG0.6 : PinSet SD_CLK : PinClear SD_CLK 
            SD_SDO = WREG0.5 : PinSet SD_CLK : PinClear SD_CLK 
            SD_SDO = WREG0.4 : PinSet SD_CLK : PinClear SD_CLK 
            SD_SDO = WREG0.3 : PinSet SD_CLK : PinClear SD_CLK 
            SD_SDO = WREG0.2 : PinSet SD_CLK : PinClear SD_CLK 
            SD_SDO = WREG0.1 : PinSet SD_CLK : PinClear SD_CLK 
            SD_SDO = WREG0.0 : PinSet SD_CLK : PinClear SD_CLK 
        EndIf 
    $endif
EndProc  

'--------------------------------------------------------------------------------------------------------------------------
' Send a command to card using SPI interface
' Input     : pCommand holds the command
'           : pValue holds the value associated with the command
' Return    :
' Notes     :
'
Proc SD_SendCmd(pCommand As Byte, pValue As Dword), Byte
    Dim wIndex As Word

$ifdef SD_Detect                                            ' Are we using the SD detect switch?
    If SD_Detect = SD_cMissing Then                         ' Is the SD insterted?
        SD_tInserted = False                                ' No. Set the flag
    EndIf
$endif
    PinSet SD_CS                                            ' Select card and wait for ready
    Low SD_CLK
    PinClear SD_CS
    
    For wIndex = 0 To 9999
        If SD_SeekResponse($FF) <> $FF Then                 ' Wait for ready
            Result = $FF
        Else 
            Break  
        EndIf           
    Next
         
    If Result = $FF Then ExitProc
    
    SD_SendByte(pCommand)                               ' Send the Command
    SD_SendByte(pValue.Byte3)                           ' \
    SD_SendByte(pValue.Byte2)                           ' | Send the value associated with the command
    SD_SendByte(pValue.Byte1)                           ' |
    SD_SendByte(pValue.Byte0)                           ' /
    If pCommand = SD_cCMD0 Then
        SD_SendByte($95)			                    ' CRC for SD_cCMD0($0000)
    ElseIf pCommand = SD_cCMD8 Then
        SD_SendByte($87)			                    ' CRC for SD_cCMD8($01AA)
    Else
        SD_SendByte($FF)                                ' Default CRC byte
    EndIf
    wIndex = 10000
    Repeat                                              ' Receive the command's response
        Result = SD_ReceiveByte()
        Dec wIndex
        If wIndex = 0 Then Break
    Until Result.7 = 0                                    ' Wait for a valid response in timeout of 100 attempts
EndProc

'--------------------------------------------------------------------------------------------------------------------------
' Sends Cmd 55 followed by 41
' Input     : pHCSBitSet
' Return    :
' Notes     :
'
Proc SD_Cmd5541(pHCSBitSet As Bit), Byte
    Result = SD_SendCmd(SD_cCMD55, 0)
    If Result > 1 Then
        ExitProc
    Else
        If pHCSBitSet = True Then
            Result = SD_SendCmd(SD_cCMD41, $40000000)
        Else
            Result = SD_SendCmd(SD_cCMD41, $00000000)
        EndIf
    EndIf
EndProc  

$define SdRaw_Init() SD_InitSequence()
    
Proc SD_InitSequence(), Bit
    Dim wIndex As Word
    Dim wOCR As Word
    Dim bSdDummy As Byte
    
    Disk_tSlowSPI = True                                                ' Set SPI to slow mode for initialisation
    
    $if SD_COM = SPI_1 Or SD_COM = SPI_2
        $if SD_COM = SPI_1
            $if McuType = 1
                SPI1STAT.15 = 0  
            $else
                SSP1CON1.5 = 0 
            $endif
        $elseif SD_COM = SPI_2
            $if McuType = 1
                SPI2STAT.15 = 0  
            $else
                SSP2CON1.5 = 0 
            $endif
        $endif   
        Output SD_CS : Input SD_SDI : Output SD_CLK : Output SD_SDO
        PinHigh SD_CS : PinLow SD_SDO : PinLow SD_CLK 
        SD_SendByte(0xFF)
    $endif
    
    PinHigh SD_CS
    PinLow SD_SDO
    PinLow SD_CLK
    PinInput SD_SDI
$ifdef SD_Detect                                                        ' Are we using an SD detect switch?
    PinInput SD_Detect                                                  ' Yes. So make it an input
    SD_tInserted = True                                                 ' Set the flag to true
$endif

$ifdef SD_Power                                                         ' Do we need to power cycle the SD card?
    PinHigh SD_Power                                                    ' Yes. So turn off the power to the SD card
    DelayMS 500                                                         ' For a while
    PinClear SD_Power                                                   ' Turn on the power to the SD card
    DelayMS 500                                                         ' Wait for stabilisation
$endif

    Result = False
    Disk_bCardType = 0
    PinSet SD_CS                                                        ' Pull SD_CS high
    PinLow SD_CLK
'
' Clock the SD for a minimum of 80 cycles
'
    wIndex = 100
    Repeat
        SD_SendByte($FF)
        Dec wIndex
    Until wIndex = 0    
     
    PinClear SD_CLK               
    PinClear SD_CS                                                      ' Pull the SD_CS low
    
    SD_SendByte(SD_cCMD0)                               ' Send the Command
    SD_SendByte(0x00)                                   ' \
    SD_SendByte(0x00)                                   ' | Send the value associated with the command
    SD_SendByte(0x00)                                   ' |
    SD_SendByte(0x00)                                   ' /
    SD_SendByte($95)			                         ' CRC for SD_cCMD0($0000)
    
    wIndex = 1000
    Repeat                                              ' Receive the command's response
        bSdDummy = SD_ReceiveByte()
        Dec wIndex
        If wIndex = 0 Then Break
    Until bSdDummy.7 = 0                                ' Wait for a valid response in timeout of 1000 attempts
    
    If bSdDummy.7 <> 0  Then ExitProc
    
    
    If SD_SendCmd(SD_cCMD8, $000001AA) = 1 Then                     ' SDC Version 2+
        SD_ReceiveByte()
        SD_ReceiveByte()
        wOCR.Byte0 = SD_ReceiveByte()
        wOCR.Byte1 = SD_ReceiveByte()
        If wOCR.Byte0 = $01 Then
            If wOCR.Byte1 = $AA Then                                ' SD Vdd range of 2.7 - 3.6 volts
                wIndex = SD_cTimeOutLimit
                Repeat
                    Dec wIndex
                    If wIndex = 0 Then Break
                Until SD_Cmd5541(True) = 0
                If wIndex > 0 Then                                  ' Cmd 55/41 successful with HCS bit set
                    If SD_SendCmd(SD_cCMD58, $00000000) = 0 Then    ' Send Cmd 58
                        wOCR.Byte0 = SD_ReceiveByte()
                        SD_ReceiveByte()                            ' \
                        SD_ReceiveByte()                            ' | Padding reads
                        SD_ReceiveByte()                            ' /
                        If wOCR.6 = 1 Then                          ' Is CCS[30] bit in the OCR set?
                            Disk_bCardType = SD_cSDHC               ' Init successful - SDHC (3)
                        Else
                            Disk_bCardType = SD_cSD                 ' Init successful - SD (2)
                        EndIf
                    EndIf
                EndIf
            EndIf
        EndIf
    Else
        If SD_Cmd5541(False) <= 1 Then
            Disk_bCardType = SD_cSD                                 ' Init successful - SD (2)
            wIndex = SD_cTimeOutLimit
            Repeat
                Dec wIndex
                If wIndex = 0 Then Break
            Until SD_Cmd5541(True) = 0                              ' Wait for leaving idle state
        Else
            Disk_bCardType = SD_cMMC                                ' Init successful - MMC (1)
            wIndex = SD_cTimeOutLimit
            Repeat
                Dec wIndex
                If wIndex = 0 Then Break
            Until SD_SendCmd(SD_cCMD1, $00) = 0                     ' Wait for leaving idle state
        EndIf
        If wIndex = 0 Then
            Disk_bCardType = 0
        EndIf
    EndIf

    
    PinSet SD_CS
    SD_SendByte($FF)                                                    ' Clock SD/MMC to complete
       
    If Disk_bCardType > 0 Then
        Result = True
        Disk_tSlowSPI = False                                           ' Set SPI to fast mode
        $if SD_COM = SPI_1 Or SD_COM = SPI_2
            $if SD_COM = SPI_1
                $if McuType = 1
                    SPI1STAT.15 = 1 
                $else
                    SSP1CON1.5 = 1 
                $endif
            $elseif SD_COM = SPI_2
                $if McuType = 1
                    SPI2STAT.15 = 1  
                $else
                    SSP2CON1.5 = 1 
                $endif
            $endif
            Output SD_CS : Input SD_SDI : Output SD_CLK : Output SD_SDO
        $endif
    EndIf
    
    If Disk_bCardType <> SD_cSDHC Then
        PinSet SD_CS 
        PinClear SD_CLK 
        PinClear SD_CS
    
        wIndex = SD_cTimeOutLimit
        $if McuType = 1
            Repeat 
                bSdDummy = SD_ReceiveByte()
                Dec wIndex
            Until bSdDummy = $FF Or wIndex = 0
        $else
            Repeat
                Dec wIndex
            Until SD_ReceiveByte() = $FF Or wIndex = 0
        $endif 
    
        SD_SendByte(SD_cCMD16)  
        SD_SendByte(0x00)
        SD_SendByte(0x00)
        SD_SendByte(0x02)
        SD_SendByte(0x00)
        SD_SendByte($FF)
    
        wIndex = SD_cTimeOutLimit
        $if McuType = 1
            Repeat 
                bSdDummy = SD_ReceiveByte()
                Dec wIndex
            Until bSdDummy = $00 Or wIndex = 0
        $else
            Repeat 
                Dec wIndex
            Until SD_ReceiveByte() = $00 Or wIndex = 0
        $endif
     
        If wIndex = 0 Then Result = False
     
        PinSet SD_CS 
     EndIf

EndProc  

$define SdRaw_ReadSector(dSector) SD_ReadSector(dSector)

Proc SD_ReadSector(dwSector As Dword), Bit
    Global Dim SectorBuffer[512] As Byte Shared
    Global Dim dwGlobalSector As Dword Shared 
    Dim wTemp As Word
    Dim bTemp As Byte
    
    PinSet SD_CS
    PinClear SD_CLK   
    PinClear SD_CS
    
    Result = True
    dwGlobalSector = 0xFFFFFFFF
     
    wTemp = 0xFFFF
    $if McuType = 1
        Repeat
            bTemp = SD_ReceiveByte()
            Dec wTemp
        Until bTemp = $FF Or wTemp = 0 
    $else
        Repeat 
            Dec wTemp
        Until SD_ReceiveByte() = $FF Or wTemp = 0
    $endif 
    
    If wTemp = 0 Then Result = False
    SD_SendByte($FF)
   
    If Disk_bCardType <> SD_cSDHC Then
        dwSector = dwSector << 9
    EndIf
    
    SD_SendByte(SD_cCMD17)  
    SD_SendByte(dwSector.Byte3)
    SD_SendByte(dwSector.Byte2)
    SD_SendByte(dwSector.Byte1)
    SD_SendByte(dwSector.Byte0)
    SD_SendByte($FF)
    
    wTemp = SD_cTimeOutLimit
    $if McuType = 1
        Repeat 
            bTemp = SD_ReceiveByte()
            Dec wTemp
        Until bTemp = $FE Or wTemp = 0
    $else
        Repeat 
            Dec wTemp
        Until SD_ReceiveByte() = $FE Or wTemp = 0
    $endif 
    If wTemp = 0 Then Result = False
    
    For wTemp = 0 To 511
        $if McuType = 1
            bTemp = SD_ReceiveByte()
            SectorBuffer[wTemp] = bTemp
        $else
            SectorBuffer[wTemp] = SD_ReceiveByte()
        $endif 
    Next
    SD_ReceiveByte() 
    SD_ReceiveByte() 
    PinSet SD_CS
EndProc

$define SdRaw_WriteSector(dSector) SD_WriteSector(dSector)

Proc SD_WriteSector(dwSector As Dword), Bit
    Global Dim SectorBuffer[512] As Byte Shared
    Global Dim dwGlobalSector As Dword Shared 
    Dim bTemp As Byte
    Dim bDummy As Byte
    Dim wTemp As Word
    
    Result = 1
    dwGlobalSector = 0xFFFFFFFF
    
    PinSet SD_CS
    PinClear SD_CLK   
    PinClear SD_CS
    
    wTemp = 0xFFFF
    $if McuType = 1
        Repeat
            bTemp = SD_ReceiveByte()
            Dec wTemp
        Until bTemp = $FF Or wTemp = 0 
    $else
        Repeat 
            Dec wTemp
        Until SD_ReceiveByte() = $FF Or wTemp = 0
    $endif 
    
    If wTemp = 0 Then Result = 0
    SD_SendByte($FF)
   
    If Disk_bCardType <> SD_cSDHC Then
        dwSector = dwSector << 9
    EndIf

    SD_SendByte(SD_cCMD24)  
    SD_SendByte(dwSector.Byte3)
    SD_SendByte(dwSector.Byte2)
    SD_SendByte(dwSector.Byte1)
    SD_SendByte(dwSector.Byte0)
    SD_SendByte($FF)
    
    wTemp = SD_cTimeOutLimit
    $if McuType = 1
        Repeat
            bTemp = SD_ReceiveByte()
            Dec wTemp
        Until bTemp = $00 Or wTemp = 0 
    $else
        Repeat 
            Dec wTemp
        Until SD_ReceiveByte() = $00 Or wTemp = 0
    $endif 
    
    If wTemp = 0 Then Result = 0
    
    SD_SendByte($FE)
    
    $if McuType = 1
        $if SD_COM = SPI_1
            SPI1BUF = SectorBuffer[0]
        $elseif SD_COM = SPI_2
            SPI2BUF = SectorBuffer[0]
        $endif
    $else 
        $if SD_COM = SPI_1
            SSP1BUF = SectorBuffer[0]
        $elseif SD_COM = SPI_2
            SSP2BUF = SectorBuffer[0]
        $endif
    $endif
    
    $if SD_COM = SPI_SW
    For wTemp = 0 To 511
    $else
    For wTemp = 1 To 511
    $endif
        $if McuType = 1
            $if SD_COM = SPI_1
                bTemp = SectorBuffer[wTemp]
                While SPI1STAT.0 = 0 : Wend
                bDummy = SPI1BUF
                SPI1BUF = bTemp
            $elseif SD_COM = SPI_2
                bTemp = SectorBuffer[wTemp]
                While SPI2STAT.0 = 0 : Wend
                bDummy = SPI2BUF
                SPI2BUF = bTemp
            $elseif SD_COM = SPI_SW
                SD_SendByte(SectorBuffer[wTemp])
            $endif
        $else    
            $if SD_COM = SPI_1
                bTemp = SectorBuffer[wTemp]
                While SSP1STAT.0 = 0 : Wend
                bDummy = SSP1BUF
                SSP1BUF = bTemp
            $elseif SD_COM = SPI_2
                bTemp = SectorBuffer[wTemp]
                While SSP2STAT.0 = 0 : Wend
                bDummy = SSP2BUF
                SSP2BUF = bTemp
            $elseif SD_COM = SPI_SW
                SD_SendByte(SectorBuffer[wTemp])
            $endif
        $endif
    Next
    
    $if McuType = 1
        $if SD_COM = SPI_1
            While SPI1STAT.0 = 0 : Wend
            bDummy = SPI1BUF
        $elseif SD_COM = SPI_2
            While SPI2STAT.0 = 0 : Wend
            bDummy = SPI2BUF
        $endif
    $else    
        $if SD_COM = SPI_1
            While SSP1STAT.0 = 0 : Wend
            bDummy = SSP1BUF
        $elseif SD_COM = SPI_2
            While SSP2STAT.0 = 0 : Wend
            bDummy = SSP2BUF
        $endif
    $endif
    
    SD_SendByte($FF)
    SD_SendByte($FF)
    
    wTemp = SD_cTimeOutLimit
    Repeat 
        bTemp = SD_ReceiveByte()
        bTemp = bTemp & $0F
        Dec wTemp
    Until bTemp = $05 Or wTemp = 0
    
    If wTemp = 0 Then Result = 0
    PinSet SD_CS
EndProc   

$define SdRaw_WriteByte(bByte, dPosition) SD_WriteByte(bByte, dPosition)

Proc SD_WriteByte(bByte As Byte, dPosition As Dword), Bit
    Global Dim SectorBuffer[512] As Byte Shared
    Global Dim dwGlobalSector As Dword Shared 
    Dim dwTemp As Dword
    Dim wTemp As dwTemp.Word0
    
    Result = 1
    
    dwTemp = dPosition >> 9
    If dwTemp <> dwGlobalSector Then 
        If SD_ReadSector(dwTemp) = 0 Then
            Result = 0  
            dwGlobalSector = 0xFFFFFFFF
            ExitProc 
        EndIf
        dwGlobalSector = dwTemp
    EndIf
    
    wTemp = dPosition // 512
    SectorBuffer[wTemp] = bByte
    If SD_WriteSector(dwGlobalSector) <> 1 Then
        Result = 0 
    EndIf
EndProc

$define SdRaw_ReadByte(dPosition) SD_ReadByte(dPosition)

Proc SD_ReadByte(dPosition As Dword), Byte
    Global Dim SectorBuffer[512] As Byte Shared
    Global Dim dwGlobalSector As Dword Shared 
    Dim dwTemp As Dword
    Dim wTemp As dwTemp.Word0
    
    Result = 1

    dwTemp = dPosition >> 9
    If dwTemp <> dwGlobalSector Then 
        If SD_ReadSector(dwTemp) = 0 Then
            Result = 0  
            dwGlobalSector = 0xFFFFFFFF
            ExitProc 
        EndIf
        dwGlobalSector = dwTemp
    EndIf
    
    wTemp = dPosition // 512
    Result = SectorBuffer[wTemp]
EndProc


























