$if TOUCH_COM = SPI_SW 
    Output T_CS : Input T_IRQ : Input T_MISO 
    Output T_MOSI : Output T_SCLK 
    PinSet T_CS
$endif

$ifndef MemoryType
    $define MemoryType 0
$endif

$ifndef FontSmooth
    $define FontSmooth 0
$endif

$if TOUCH_IC = XPT2046
    $define X_Command_Byte      0xD1 'Touch screen - X values
    $define Y_Command_Byte      0x90 'Touch screen - Y values
    $define Z1_Command_Byte     0xB1 'Touch screen - Z1 value
    $define Z2_Command_Byte     0xC1 'Touch screen - Z2 value
    $define TouchADC            4095 'Touch screen - ADC resolution
$endif

$if _core >= 24
    Symbol TftTouchDelayCs = ((3 * _xtal) / 50)
$else
    Symbol TftTouchDelayCs = ((3 * _xtal) / 100) 
$endif

Proc _______TFT_TOUCH_____()
EndProc

'Global variables to be used by the user program:
'wTouchToPointX, wTouchToPointY - hold converted touch coordinates to actual screen point (pixel)
'fKX1, fKX2, fKX3, fKY1, fKY2, fKY3 - these are the conversion (calibration) coefficients used to convert touch coordinates to actual 
'                                     screen point (pixel). These can be obtained by running the calibration procedure TftTouchCalibration(N) 
'                                     and used or saved in memory for later use.


$if McuType = 0
Dim wTftTemp1_Touch As Word       At sTftString#0   
Dim wTftTemp2_Touch As Word       At sTftString#2   
Dim waPointCalX[9] As Word        At sTftString#4   
Dim bTftCounter0_Touch As Byte    At sTftString#22  
Dim bTftCounter1_Touch As Byte    At sTftString#23  
Dim wTftTemp3_Touch As Word       At sTftString#24  
Dim wTftTemp4_Touch As Word       At sTftString#26  
Dim wDummy1 As Word   	          At sTftString#28  
Dim wDummy2 As Word               At sTftString#40  
Dim wDummy3 As Word               At sTftString#42  
Dim waPointCalY[9] As Word        At sTftString#44  
Dim fTftTemp14_Touch As Float     At sTftString#0   
Dim fTftTemp4_Touch As Float      At sTftString#4   
Dim fTftTemp5_Touch As Float      At sTftString#8  
Dim fTftTemp6_Touch As Float      At sTftString#12 
Dim fTftTemp7_Touch As Float      At sTftString#16 
Dim fTftTemp8_Touch As Float      At sTftString#20 
Dim fTftTemp16_Touch As Float     At sTftString#24  
Dim fTftTemp1_Touch As Float	  At sTftString#28  
Dim fTftTemp2_Touch As Float	  At sTftString#32  
Dim dwTftTemp1_Touch As Dword     At sTftString#36  
Dim dwTftTemp2_Touch As Dword     At sTftString#40  
Dim fTftTemp9_Touch As Float      At sTftString#44  
Dim fTftTemp10_Touch As Float     At sTftString#48  
Dim fTftTemp11_Touch As Float     At sTftString#52  
Dim fTftTemp12_Touch As Float     At sTftString#56  
Dim fTftTemp13_Touch As Float     At sTftString#60  
Dim fTftTemp3_Touch As Float      At sTftString#64  
Dim fTftTemp15_Touch As Float     At sTftString#68  
$else
Dim wTftTemp1_Touch As Word       At sTftString_0   
Dim wTftTemp2_Touch As Word       At sTftString_2   
Dim waPointCalX[9] As Word        At sTftString_4   
Dim bTftCounter0_Touch As Byte    At sTftString_22  
Dim bTftCounter1_Touch As Byte    At sTftString_23  
Dim wTftTemp3_Touch As Word       At sTftString_24  
Dim wTftTemp4_Touch As Word       At sTftString_26  
Dim wDummy1 As Word   	          At sTftString_28  
Dim wDummy2 As Word               At sTftString_40  
Dim wDummy3 As Word               At sTftString_42  
Dim waPointCalY[9] As Word        At sTftString_44  
Dim fTftTemp14_Touch As Float     At sTftString_0   
Dim fTftTemp4_Touch As Float      At sTftString_4   
Dim fTftTemp5_Touch As Float      At sTftString_8  
Dim fTftTemp6_Touch As Float      At sTftString_12 
Dim fTftTemp7_Touch As Float      At sTftString_16 
Dim fTftTemp8_Touch As Float      At sTftString_20 
Dim fTftTemp16_Touch As Float     At sTftString_24  
Dim fTftTemp1_Touch As Float	  At sTftString_28  
Dim fTftTemp2_Touch As Float	  At sTftString_32  
Dim dwTftTemp1_Touch As Dword     At sTftString_36  
Dim dwTftTemp2_Touch As Dword     At sTftString_40  
Dim fTftTemp9_Touch As Float      At sTftString_44  
Dim fTftTemp10_Touch As Float     At sTftString_48  
Dim fTftTemp11_Touch As Float     At sTftString_52  
Dim fTftTemp12_Touch As Float     At sTftString_56  
Dim fTftTemp13_Touch As Float     At sTftString_60  
Dim fTftTemp3_Touch As Float      At sTftString_64  
Dim fTftTemp15_Touch As Float     At sTftString_68  
$endif


'This procedure will return the Touch-to-Screen equivalent coordinates
'The coordinates are being placed in two global variables: wTouchToPointX and wTouchToPointY
'pMode: 1 = use Low Pass filter for touch ADC readouts which is slower but more accurate
'       0 = do not use Low Pass filter for touch ADC readouts which is faster but less accurate
'Return parameter indicates if the readouts are within the screen width and height, useful for border-line readouts

Proc TftTouchGetCoordinates(pMode As Bit), Bit
    Result = 0
    If TftTouchScreenReadValues() = 1 Then
        If pMode = 1 Then
            TftTouchReadPointAverage() 
            If TftTouchCalcPoint(wTftTouchX, wTftTouchY) = 1 Then
                Result = 1  
            EndIf
        Else
            If TftTouchCalcPoint(wTftTouchX, wTftTouchY) = 1 Then
                Result = 1  
            EndIf
        EndIf
    Else
        Result = 0
        ExitProc
    EndIf
EndProc

'Average and Low Pass filter the readouts

Proc TftTouchReadPointAverage() 
    Dim waX[TouchAveragePoints] As SWord
    Dim waY[TouchAveragePoints] As SWord
    Dim bTemp0 As Byte At dwDummy2.Byte2
    Dim bTemp1 As Byte At dwDummy2.Byte3
    Dim wMedX As Word
    Dim wMedY As Word
    Dim bTemp As Byte 
    
    
    bTemp0 = TouchAveragePoints 
    While bTemp0 > 0
        If TftTouchScreenReadValues() = 1 Then
            bTemp = bTemp0 - 1  
            waX[bTemp] = wTftTouchX 
            waY[bTemp] = wTftTouchY 
            Dec bTemp0
        EndIf     
    Wend
    
    wMedX = TftMedian16(waX, TouchAveragePoints)
    wMedY = TftMedian16(waY, TouchAveragePoints)

    bTemp0 = 0 : wTftTouchX = 0 : wTftTouchY = 0
    For bTemp1 = 0 To TouchAveragePoints - 1
        If Abs(waX[bTemp1] - wMedX) < TouchMedianDistance Then
            If Abs(waY[bTemp0] - wMedY) < TouchMedianDistance Then
                wTftTouchX = wTftTouchX + waX[bTemp1]
                wTftTouchY = wTftTouchY + waY[bTemp1]
                Inc bTemp0
            EndIf   
        EndIf        
    Next
    
    If bTemp0 > 0 Then
        wTftTouchX = wTftTouchX / bTemp0
        wTftTouchY = wTftTouchY / bTemp0
    Else
        wTftTouchX = 0
        wTftTouchY = 0
    EndIf
EndProc

'Les' buble sort
Proc TftMedian16(ByRef pArrayAddr As Word, pArraySize As Byte), Word
    Dim tSwap_Occured As Byte                                    
    Dim bElementCount As Byte                                   
    Dim wElement_0    As Word                                   
    Dim wElement_1    As Word                                   
    Dim wArrayAddress As Word                                  

    Repeat                                                      
        wArrayAddress = pArrayAddr                              
        tSwap_Occured = 0                                       
        bElementCount = 1                                       
        Repeat                                                  
            wElement_0 = Ptr16(wArrayAddress++)                 
            wElement_1 = Ptr16(wArrayAddress--)                 
            If wElement_0 > wElement_1 Then                     
                Ptr16(wArrayAddress++) = wElement_1             
                Ptr16(wArrayAddress) = wElement_0               
                tSwap_Occured = 1                              
            Else                                                
                wArrayAddress = wArrayAddress + 2              
            EndIf
            Inc bElementCount                                   
        Until bElementCount = pArraySize                       
    Until tSwap_Occured = 0                                     
    pArraySize = pArraySize / 2                                 
    pArraySize = pArraySize * 2                                
    wArrayAddress = pArrayAddr + pArraySize                    
    Result = Ptr16(wArrayAddress)                              
EndProc

'Read and convert data from touch screen ADC

Proc TftTouchScreenReadValues(), Bit '0 = invalid, 1 = valid
    Global Dim fKX1 As Float Shared
    Global Dim fKX2 As Float Shared
    Global Dim fKX3 As Float Shared
    Global Dim fKY1 As Float Shared
    Global Dim fKY2 As Float Shared
    Global Dim fKY3 As Float Shared
    
    Dim wTemp1 As Word At dwDummy3.Word0
    Dim wTemp2 As Word At dwDummy3.Word1
    
    $if TFT_COM = SPI_1
        $if _TFT_MOSI = _T_MOSI Or _TFT_MISO = _T_MISO Or _TFT_SCLK = _T_SCLK
            $if McuType = 0
                SSP1CON1.5 = 0 
            $else
                SPI1STAT.15 = 0 
            $endif
        $endif
    $endif
    
    $if TFT_COM = SPI_2
        $if _TFT_MOSI = _T_MOSI Or _TFT_MISO = _T_MISO Or _TFT_SCLK = _T_SCLK
            $if McuType = 0
                SSP2CON1.5 = 0 
            $else
                SPI2STAT.15 = 0 
            $endif
        $endif
    $endif
    
    $if SDCardUsed = 1
        $if SD_COM = SPI_1
            $if _SD_SDO = _T_MOSI Or _SD_SDI = _T_MISO Or _SD_CLK = _T_SCLK
                $if McuType = 0
                    SSP1CON1.5 = 0 
                $else
                    SPI1STAT.15 = 0 
                $endif
            $endif
        $endif
        
        $if SD_COM = SPI_2
            $if _SD_SDO = _T_MOSI Or _SD_SDI = _T_MISO Or _SD_CLK = _T_SCLK
                $if McuType = 0
                    SSP2CON1.5 = 0 
                $else
                    SPI2STAT.15 = 0 
                $endif
            $endif
        $endif
    $endif
    
    $if MemoryType <> 0
        $if MEM_COM = SPI_1
            $if _MEM_MOSI = _T_MOSI Or _MEM_MISO = _T_MISO Or _MEM_SCLK = _T_SCLK
                $if McuType = 0
                    SSP1CON1.5 = 0 
                $else
                    SPI1STAT.15 = 0 
                $endif
            $endif
        $endif
        
        $if MEM_COM = SPI_2
            $if _MEM_MOSI = _T_MOSI Or _MEM_MISO = _T_MISO Or _MEM_SCLK = _T_SCLK
                $if McuType = 0
                    SSP2CON1.5 = 0 
                $else
                    SPI2STAT.15 = 0 
                $endif
            $endif
        $endif
    $endif
    
    PinClear T_SCLK : PinClear T_CS
        TftSpiSendTs(Z1_Command_Byte) 
        DelayUS 2
        PinClear T_MOSI
        wTemp1.Byte1 = TftSpiReceiveTs()
        wTemp1.Byte0 = TftSpiReceiveTs()
        TftSpiSendTs(Z2_Command_Byte)
        DelayUS 2
        PinClear T_MOSI
        wTemp2.Byte1 = TftSpiReceiveTs()
        wTemp2.Byte0 = TftSpiReceiveTs()
    PinSet T_CS
    wTemp1 = wTemp1 >> 3
    wTemp2 = wTemp2 >> 3
    wTftTouchZ = wTemp1 + 4095 
    wTftTouchZ = wTftTouchZ - wTemp2
    
    If wTftTouchZ >= Z_Threshold Then 
        PinClear T_SCLK : PinClear T_CS
            TftSpiSendTs(X_Command_Byte)
            DelayUS 2
            PinClear T_MOSI
            wTftTouchX.Byte1 = TftSpiReceiveTs()
            wTftTouchX.Byte0 = TftSpiReceiveTs() 
            TftSpiSendTs(X_Command_Byte)
            DelayUS 2
            PinClear T_MOSI
            wTftTouchX.Byte1 = TftSpiReceiveTs()
            wTftTouchX.Byte0 = TftSpiReceiveTs()
            TftSpiSendTs(Y_Command_Byte)
            DelayUS 2
            PinClear T_MOSI
            wTftTouchY.Byte1 = TftSpiReceiveTs()
            wTftTouchY.Byte0 = TftSpiReceiveTs()
        PinSet T_CS
        wTftTouchX = wTftTouchX >> 3
        wTftTouchY = wTftTouchY >> 3
    Else
        PinClear T_SCLK : PinClear T_CS 
            TftSpiSendTs(Y_Command_Byte)
            DelayUS 2
            PinClear T_MOSI
            wTftTouchY.Byte1 = TftSpiReceiveTs()
            wTftTouchY.Byte0 = TftSpiReceiveTs() 
        PinSet T_CS  
        wTftTouchX = 0
        wTftTouchY = 0
    EndIf  

    $if TFT_COM = SPI_1
        $if _TFT_MOSI = _T_MOSI Or _TFT_MISO = _T_MISO Or _TFT_SCLK = _T_SCLK
            $if McuType = 0
                SSP1CON1.5 = 1 
            $else
                SPI1STAT.15 = 1 
            $endif
        $endif
    $endif
    
    $if TFT_COM = SPI_2
        $if _TFT_MOSI = _T_MOSI Or _TFT_MISO = _T_MISO Or _TFT_SCLK = _T_SCLK
            $if McuType = 0
                SSP2CON1.5 = 1 
            $else
                SPI2STAT.15 = 1 
            $endif
        $endif
    $endif
    
    $if SDCardUsed = 1
        $if SD_COM = SPI_1
            $if _SD_SDO = _T_MOSI Or _SD_SDI = _T_MISO Or _SD_CLK = _T_SCLK
                $if McuType = 0
                    SSP1CON1.5 = 1 
                $else
                    SPI1STAT.15 = 1 
                $endif
            $endif
        $endif
        
        $if SD_COM = SPI_2
            $if _SD_SDO = _T_MOSI Or _SD_SDI = _T_MISO Or _SD_CLK = _T_SCLK
                $if McuType = 0
                    SSP2CON1.5 = 1 
                $else
                    SPI2STAT.15 = 1 
                $endif
            $endif
        $endif
    $endif
    
    $if MemoryType <> 0
        $if MEM_COM = SPI_1
            $if _MEM_MOSI = _T_MOSI Or _MEM_MISO = _T_MISO Or _MEM_SCLK = _T_SCLK
                $if McuType = 0
                    SSP1CON1.5 = 1 
                $else
                    SPI1STAT.15 = 1 
                $endif
            $endif
        $endif
        
        $if MEM_COM = SPI_2
            $if _MEM_MOSI = _T_MOSI Or _MEM_MISO = _T_MISO Or _MEM_SCLK = _T_SCLK
                $if McuType = 0
                    SSP2CON1.5 = 1 
                $else
                    SPI2STAT.15 = 1 
                $endif
            $endif
        $endif
    $endif
    
    Result = 0
    If wTftTouchZ > Z_Threshold Then
        If wTftTouchX < TouchADC - 10 Then
            If wTftTouchZ < TouchADC - 10 Then
                If wTftTouchY < TouchADC - 10 Then
                    If wTftTouchX > 10 Then  
                        If wTftTouchY > 10 Then
                            If wTftTouchZ > 10 Then
                                Result = 1
                            EndIf
                        EndIf
                    EndIf
                EndIf
            EndIf
        EndIf
    EndIf
EndProc

'This procedure initializes a touch screen calibration process
'bPoints = number of point to be used for calibration, allowed values: 3, 5, 7, 9
'Note: higher number of calibration points gives better calibration Result but takes longer to complete 
'To complete the calibration process tap 5 times on each marked point until it disappears

Proc TftTouchCalibration(bPoints As Byte)
    TftTouchEnableIrq()
    TftResetScreen(BLACK)
    TftTouchDrawPoints(bPoints)
    TftTouchTestPoints(bPoints)
    TftTouchCof(bPoints)
EndProc

'Draws the calibration points on the TFT screen to be pressed

Proc TftTouchDrawPoints(bPoints As Byte)
    If bPoints > 2 Then
        TftTouchCalcPointCenter(0)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, MAGENTA)
        TftTouchCalcPointCenter(1)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, RED)
        TftTouchCalcPointCenter(2)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, GREEN)
    EndIf
    
    If bPoints > 4 Then
        TftTouchCalcPointCenter(3)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, WHITE)    
        TftTouchCalcPointCenter(4)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, OLIVE)
    EndIf
    
    If bPoints > 6 Then
        TftTouchCalcPointCenter(5)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, BLUE)  
        TftTouchCalcPointCenter(6)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, CYAN)
    EndIf
    
    If bPoints > 8 Then
        TftTouchCalcPointCenter(7)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, PURPLE) 
        TftTouchCalcPointCenter(8)
        TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 4, PINK)  
    EndIf
EndProc

'Calculates the calibration point center coordinates relative to screen width and height

Proc TftTouchCalcPointCenter(bPoint As Byte)
    Select bPoint
        Case 0
            wTftTemp1_Touch = wTftScreenWidth - (wTftScreenWidth / 10)
            wTftTemp2_Touch = wTftScreenHeight / 10
        Case 1
            wTftTemp1_Touch = wTftScreenWidth - (wTftScreenWidth / 8)
            wTftTemp2_Touch = wTftScreenHeight - (wTftScreenHeight / 5)
        Case 2
            wTftTemp1_Touch = wTftScreenWidth / 7
            wTftTemp2_Touch = wTftScreenHeight / 2      
        Case 3  
            wTftTemp1_Touch = wTftScreenWidth - 3 * (wTftScreenWidth / 5)
            wTftTemp2_Touch = wTftScreenHeight / 7
        Case 4
            wTftTemp1_Touch = wTftScreenWidth / 2
            wTftTemp2_Touch = wTftScreenHeight - (wTftScreenHeight / 6)
        Case 5
            wTftTemp1_Touch = wTftScreenWidth / 12
            wTftTemp2_Touch = wTftScreenHeight - (wTftScreenHeight / 9)
        Case 6
            wTftTemp1_Touch = wTftScreenWidth / 13
            wTftTemp2_Touch = wTftScreenHeight / 8
        Case 7
            wTftTemp1_Touch = 2 * (wTftScreenWidth / 5)
            wTftTemp2_Touch = 3 * (wTftScreenHeight / 5)
        Case 8
            wTftTemp1_Touch = 4 * (wTftScreenWidth / 5)
            wTftTemp2_Touch = 2 * (wTftScreenHeight / 5)
    EndSelect        
EndProc

'Marks each test point at focus with a circle around it

Proc TftTouchSetPoint(bPoint As Byte)
    TftTouchCalcPointCenter(bPoint)
    Select bPoint
        Case 0
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, MAGENTA) 
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, MAGENTA)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, MAGENTA)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, MAGENTA)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, MAGENTA)
        Case 1
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, RED) 
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, RED)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, RED)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, RED)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, RED)
        Case 2
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, GREEN)
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, GREEN)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, GREEN)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, GREEN)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, GREEN)
        Case 3
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, WHITE) 
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, WHITE)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, WHITE)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, WHITE)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, WHITE)
        Case 4
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, OLIVE)  
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, OLIVE)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, OLIVE)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, OLIVE)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, OLIVE)   
        Case 5
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, BLUE) 
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, BLUE)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, BLUE)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, BLUE)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, BLUE)
        Case 6
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, CYAN)
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, CYAN)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, CYAN)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, CYAN)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, CYAN)
        Case 7
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, PURPLE)
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, PURPLE)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, PURPLE)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, PURPLE)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, PURPLE)
        Case 8
            'TftCircleQE(wTftTemp1_Touch, wTftTemp2_Touch, 8, PINK)
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, PINK)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, PINK)
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, PINK)
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, PINK) 
    EndSelect
EndProc

'Removes tested points

Proc TftTouchRemovePoint(bPoint As Byte, bStep As Byte)
    TftTouchCalcPointCenter(bPoint)
    Select bStep
        Case 5
            TftLineDNF(wTftTemp1_Touch - 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch + 8, BLACK)
        Case 4
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch + 8, wTftTemp1_Touch + 8, wTftTemp2_Touch, BLACK)
        Case 3
            TftLineDNF(wTftTemp1_Touch + 8, wTftTemp2_Touch, wTftTemp1_Touch, wTftTemp2_Touch - 8, BLACK)
        Case 2
            TftLineDNF(wTftTemp1_Touch, wTftTemp2_Touch - 8, wTftTemp1_Touch - 8, wTftTemp2_Touch, BLACK)
        Case 1
            TftCircleQS(wTftTemp1_Touch, wTftTemp2_Touch, 8, BLACK) 
    EndSelect
EndProc

'Test each calibration point 

Proc TftTouchTestPoints(bPoints As Byte)
    For bTftCounter0_Touch = 0 To bPoints - 1
        DelayMS 500
        TftTouchSetPoint(bTftCounter0_Touch)
        wTftTemp3_Touch = 0
        wTftTemp4_Touch = 0
        For bTftCounter1_Touch = 5 DownTo 1
            DelayMS 500
            Do
                If T_IRQ = 0 Then 
                    If TftTouchScreenReadValues() = 1 Then
                        TftTouchReadPointAverage() 
                        If wTftTouchX <> 0 Then
                            If wTftTouchY <> 0 Then
                                wTftTemp3_Touch = wTftTemp3_Touch + wTftTouchX 
                                wTftTemp4_Touch = wTftTemp4_Touch + wTftTouchY
                                TftTouchRemovePoint(bTftCounter0_Touch, bTftCounter1_Touch)
                                DelayMS 250
                                Break
                            EndIf
                        EndIf
                    EndIf
                EndIf
            Loop
        Next
        waPointCalX[bTftCounter0_Touch] = wTftTemp3_Touch / 5
        waPointCalY[bTftCounter0_Touch] = wTftTemp4_Touch / 5
    Next
EndProc

'Compute calibration coefficients

Proc TftTouchCof(bPoint As Byte)
    Dim dwTftTemp3_Touch As Dword At fKX1 
    Dim dwTftTemp4_Touch As Dword At fKX2 
    Dim dwTftTemp5_Touch As Dword At fKX3 
    Dim dwTftTemp6_Touch As Dword At fKY1 
    Dim dwTftTemp7_Touch As Dword At fKY2 
    Dim dwTftTemp8_Touch As Dword At fKY3 
    Dim dwTftTemp9_Touch As Dword At dwDummy1 
    Dim dwTftTemp10_Touch As Dword At dwDummy2 
    Dim dwTftTemp11_Touch As Dword At dwDummy3 
    
       

    dwTftTemp1_Touch = 0 : dwTftTemp2_Touch = 0 : dwTftTemp3_Touch = 0 : dwTftTemp4_Touch = 0 : dwTftTemp5_Touch = 0
    dwTftTemp6_Touch = 0 : dwTftTemp7_Touch = 0 : dwTftTemp8_Touch = 0 : dwTftTemp9_Touch = 0 : dwTftTemp10_Touch = 0 : dwTftTemp11_Touch = 0 
    For bTftCounter0_Touch = 0 To bPoint - 1
        TftTouchCalcPointCenter(bTftCounter0_Touch)
        dwTftTemp1_Touch  = dwTftTemp1_Touch  + waPointCalX[bTftCounter0_Touch] * waPointCalX[bTftCounter0_Touch]
        dwTftTemp2_Touch = dwTftTemp2_Touch + waPointCalX[bTftCounter0_Touch] * waPointCalY[bTftCounter0_Touch]  
        dwTftTemp3_Touch  = dwTftTemp3_Touch  + waPointCalY[bTftCounter0_Touch] * waPointCalY[bTftCounter0_Touch]
        dwTftTemp4_Touch  = dwTftTemp4_Touch  + waPointCalX[bTftCounter0_Touch] * wTftTemp1_Touch 
        dwTftTemp5_Touch  = dwTftTemp5_Touch  + waPointCalY[bTftCounter0_Touch] * wTftTemp1_Touch 
        dwTftTemp6_Touch  = dwTftTemp6_Touch  + waPointCalX[bTftCounter0_Touch] * wTftTemp2_Touch 
        dwTftTemp7_Touch  = dwTftTemp7_Touch  + waPointCalY[bTftCounter0_Touch] * wTftTemp2_Touch 
        dwTftTemp8_Touch   = dwTftTemp8_Touch   + waPointCalX[bTftCounter0_Touch]
        dwTftTemp9_Touch   = dwTftTemp9_Touch   + waPointCalY[bTftCounter0_Touch]
        dwTftTemp10_Touch   = dwTftTemp10_Touch   + wTftTemp1_Touch
        dwTftTemp11_Touch   = dwTftTemp11_Touch   + wTftTemp2_Touch
    Next
    
    fTftTemp5_Touch = dwTftTemp1_Touch / dwTftTemp8_Touch
    fTftTemp6_Touch = dwTftTemp2_Touch / dwTftTemp9_Touch
    fTftTemp7_Touch = dwTftTemp8_Touch / bPoint
    fTftTemp8_Touch = dwTftTemp2_Touch / dwTftTemp8_Touch
    fTftTemp9_Touch = dwTftTemp3_Touch / dwTftTemp9_Touch
    fTftTemp10_Touch = dwTftTemp9_Touch / bPoint
    fTftTemp1_Touch = fTftTemp5_Touch - fTftTemp7_Touch
    fTftTemp1_Touch = fTftTemp1_Touch * (fTftTemp9_Touch - fTftTemp10_Touch)
    fTftTemp2_Touch = (fTftTemp6_Touch - fTftTemp7_Touch) 
    fTftTemp2_Touch = fTftTemp2_Touch * (fTftTemp8_Touch - fTftTemp10_Touch)
    fTftTemp4_Touch = fTftTemp1_Touch - fTftTemp2_Touch
    fTftTemp11_Touch = dwTftTemp4_Touch / dwTftTemp8_Touch
    fTftTemp12_Touch = dwTftTemp5_Touch / dwTftTemp9_Touch
    fTftTemp13_Touch = dwTftTemp10_Touch / bPoint
    fTftTemp14_Touch = dwTftTemp6_Touch / dwTftTemp8_Touch
    fTftTemp15_Touch = dwTftTemp7_Touch / dwTftTemp9_Touch
    fTftTemp16_Touch = dwTftTemp11_Touch / bPoint
    
    fTftTemp1_Touch = fTftTemp11_Touch - fTftTemp13_Touch
    fTftTemp1_Touch = fTftTemp1_Touch * (fTftTemp9_Touch - fTftTemp10_Touch)
    fTftTemp2_Touch = fTftTemp12_Touch - fTftTemp13_Touch
    fTftTemp2_Touch = fTftTemp2_Touch * (fTftTemp8_Touch - fTftTemp10_Touch)
    fKX1 = fTftTemp1_Touch - fTftTemp2_Touch
    fKX1 = fKX1 / fTftTemp4_Touch
    
    fTftTemp1_Touch = fTftTemp12_Touch - fTftTemp13_Touch
    fTftTemp1_Touch = fTftTemp1_Touch * (fTftTemp5_Touch - fTftTemp7_Touch)
    fTftTemp2_Touch = fTftTemp11_Touch - fTftTemp13_Touch
    fTftTemp2_Touch = fTftTemp2_Touch * (fTftTemp6_Touch - fTftTemp7_Touch)
    fKX2 = fTftTemp1_Touch - fTftTemp2_Touch
    fKX2 = fKX2 / fTftTemp4_Touch
    
    fTftTemp1_Touch = fTftTemp14_Touch - fTftTemp16_Touch
    fTftTemp1_Touch = fTftTemp1_Touch * (fTftTemp9_Touch - fTftTemp10_Touch)
    fTftTemp2_Touch = fTftTemp15_Touch - fTftTemp16_Touch
    fTftTemp2_Touch = fTftTemp2_Touch * (fTftTemp8_Touch - fTftTemp10_Touch)
    fKY1 = fTftTemp1_Touch - fTftTemp2_Touch
    fKY1 = fKY1 / fTftTemp4_Touch
    
    fTftTemp1_Touch = fTftTemp15_Touch - fTftTemp16_Touch
    fTftTemp1_Touch = fTftTemp1_Touch * (fTftTemp5_Touch - fTftTemp7_Touch)
    fTftTemp2_Touch = fTftTemp14_Touch - fTftTemp16_Touch
    fTftTemp2_Touch = fTftTemp2_Touch * (fTftTemp6_Touch - fTftTemp7_Touch)
    fKY2 = fTftTemp1_Touch - fTftTemp2_Touch
    fKY2 = fKY2 / fTftTemp4_Touch
    
    fTftTemp1_Touch = fTftTemp7_Touch * fTftTemp12_Touch - fTftTemp6_Touch * fTftTemp13_Touch
    fTftTemp1_Touch = fTftTemp1_Touch * fTftTemp8_Touch
    fTftTemp2_Touch = fTftTemp5_Touch * fTftTemp13_Touch - fTftTemp7_Touch * fTftTemp11_Touch
    fTftTemp2_Touch = fTftTemp2_Touch * fTftTemp9_Touch
    fTftTemp3_Touch = fTftTemp6_Touch * fTftTemp11_Touch - fTftTemp5_Touch * fTftTemp12_Touch
    fTftTemp3_Touch = fTftTemp3_Touch * fTftTemp10_Touch
    fKX3 = fTftTemp1_Touch + fTftTemp2_Touch 
    fKX3 = fKX3 + fTftTemp3_Touch
    fKX3 = fKX3 / fTftTemp4_Touch
    
    fTftTemp1_Touch = fTftTemp7_Touch * fTftTemp15_Touch - fTftTemp6_Touch * fTftTemp16_Touch
    fTftTemp1_Touch = fTftTemp1_Touch * fTftTemp8_Touch
    fTftTemp2_Touch = fTftTemp5_Touch * fTftTemp16_Touch - fTftTemp7_Touch * fTftTemp14_Touch
    fTftTemp2_Touch = fTftTemp2_Touch * fTftTemp9_Touch
    fTftTemp3_Touch = fTftTemp6_Touch * fTftTemp14_Touch - fTftTemp5_Touch * fTftTemp15_Touch
    fTftTemp3_Touch = fTftTemp3_Touch * fTftTemp10_Touch
    fKY3 = fTftTemp1_Touch + fTftTemp2_Touch 
    fKY3 = fKY3 + fTftTemp3_Touch
    fKY3 = fKY3 / fTftTemp4_Touch
EndProc

'Calculate screen coordinates from touch ADC

Proc TftTouchCalcPoint(wTcX As Word, wTcY As Word), Bit
    Global Dim wTouchToPointX As Word Shared 
    Global Dim wTouchToPointY As Word Shared 
    
    Dim fTemp1_Touch As Float At dwDummy2
    Dim fTemp2_Touch As Float At dwDummy3
    
    fTemp1_Touch = fKX1 * wTcX
    fTemp2_Touch = fKX2 * wTcY
    fTemp1_Touch = fTemp1_Touch + fTemp2_Touch
    fTemp2_Touch = fTemp1_Touch + fKX3
    wTouchToPointX = fTemp2_Touch
    fTemp1_Touch = fKY1 * wTcX
    fTemp2_Touch = fKY2 * wTcY
    fTemp1_Touch = fTemp1_Touch + fTemp2_Touch
    fTemp2_Touch = fTemp1_Touch + fKY3
    wTouchToPointY = fTemp2_Touch    
    
    Result = 0
    If wTouchToPointX < wTftScreenWidth - 1 And wTouchToPointX > 0 Then
        If wTouchToPointY < wTftScreenHeight - 1 And wTouchToPointY > 0 Then
            Result = 1
        EndIf 
    EndIf
EndProc

'Touch screen enable T_IRQ pin interrupt
Proc TftTouchEnableIrq()
    TftTouchScreenReadValues()
EndProc

'Touch screen software SPI send

Proc TftSpiSendTs(bSpiByte As Byte)
    Dim bTemp As Byte

    For bTemp = 7 DownTo 0 Step -1
        T_MOSI = GetBit bSpiByte, bTemp
        DelayCS TftTouchDelayCs : PinSet T_SCLK : DelayCS TftTouchDelayCs : PinClear T_SCLK : DelayCS TftTouchDelayCs   
    Next
EndProc

'Touch screen software SPI receive

Proc TftSpiReceiveTs(), Byte
    Dim bTemp As Byte

    For bTemp = 7 DownTo 0 Step -1
        DelayCS TftTouchDelayCs : PinSet T_SCLK : DelayCS TftTouchDelayCs
        LoadBit Result, bTemp, T_MISO
        PinClear T_SCLK : DelayCS TftTouchDelayCs
    Next
EndProc
